Package: GOstats
Title: Tools for manipulating GO and microarrays
Version: 2.77.0
Authors@R: c(
        person("Robert", "Gentleman", role="aut"),
        person("Seth", "Falcon", role="ctb"),
        person("Robert", "Castelo", role="ctb"),
        person("Sonali", "Kumari", role="ctb",
                comment="Converted vignettes from Sweave to R Markdown / HTML."),
        person("Dennis", "Ndubi", role="ctb",
                comment="Converted GOstatsHyperG vignette from Sweave to R Markdown / HTML."),
        person("Bioconductor Package Maintainer", role="cre",
                email="maintainer@bioconductor.org"))
Description: A set of tools for interacting with GO and microarray
        data. A variety of basic manipulation tools for graphs,
        hypothesis testing and other simple calculations.
Depends: R (>= 2.10), Biobase (>= 1.15.29), Category (>= 2.43.2), graph
Imports: methods, stats, stats4, AnnotationDbi (>= 0.0.89), GO.db (>=
        1.13.0), RBGL, annotate (>= 1.13.2), AnnotationForge, Rgraphviz
Suggests: hgu95av2.db (>= 1.13.0), ALL, multtest, genefilter,
        RColorBrewer, xtable, SparseM, GSEABase, geneplotter,
        org.Hs.eg.db, RUnit, BiocGenerics, BiocStyle, knitr
License: Artistic-2.0
Collate: zzz.R AllClasses.R AllGenerics.R hyperGTest-methods.R
        GOHyperGResult-accessors.R GOgraph.R GOhyptest.R hyperGtable.R
        shortestPath.R triad.R OBOHyperGResult-accessors.R
biocViews: Annotation, GO, MultipleComparison, GeneExpression,
        Microarray, Pathways, GeneSetEnrichment, GraphAndNetwork
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/GOstats
git_branch: devel
git_last_commit: d09aa16
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 22:18:51 UTC; biocbuild
Author: Robert Gentleman [aut],
  Seth Falcon [ctb],
  Robert Castelo [ctb],
  Sonali Kumari [ctb] (Converted vignettes from Sweave to R Markdown /
    HTML.),
  Dennis Ndubi [ctb] (Converted GOstatsHyperG vignette from Sweave to R
    Markdown / HTML.),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>

import(methods)
importFrom(stats, cor, setNames)
importFrom(stats4, summary)

import(Category)
import(graph)

importMethodsFrom(AnnotationDbi,
                  Term,
                  eapply,
                  ls,
                  mget,
                  nrow,
                  revmap)

importMethodsFrom(Biobase,
                  annotation,
                  exprs,
                  featureNames)

importFrom(AnnotationDbi,
           make_eg_to_go_map)

importFrom(AnnotationForge,
           available.dbschemas)

importFrom(Biobase,
           addVigs2WinMenu,
           subListExtract)

importFrom(RBGL,
           connectedComp,
           dijkstra.sp,
           sp.between)

importFrom(annotate,
           dropECode,
           getAnnMap,
           getOntology)

importFrom(GO.db,
           GOTERM)

importFrom(Rgraphviz,
           makeNodeAttrs)

importMethodsFrom(Rgraphviz,
                  plot)

export(compCorrGraph, compGdist,
       enumPairs, 
       GOGraph,
       idx2dimnames, isTriad,
       makeGOGraph, notConn, oneGOGraph,
       reduce2Degreek, shortestPath, simLL,
       simLP, simUI, triadCensus,
       probeSetSummary)

exportClasses(GOHyperGResult)

exportMethods("hyperGTest",
              "pvalues",
              "description",
              "oddsRatios",
              "expectedCounts",
              "htmlReport",
              "goDag",
              "geneCounts",
              "universeCounts",
              "geneIds",
              "geneIdUniverse",
              "universeMappedCount",
              "geneMappedCount",
              "annotation",
              "testName",
              "summary")

export(termGraphs, 
       inducedTermGraph, 
       plotGOTermGraph)

resultToGOHyperG <- function(res, origIds) {
    go2Affy <- mget(names(pvalues(res)),
                    getAnnMap("GO2ALLPROBES", annotation(res)))
    list(pvalues=pvalues(res),
         goCounts=universeCounts(res),
         intCounts=geneCounts(res),
         numLL=universeMappedCount(res),
         numInt=geneMappedCount(res),
         chip=annotation(res),
         intLLs=origIds,
         go2Affy=go2Affy)
}

## this is for OBO
## the function orderedAttr() is defined
## in GOHyperGResults-accessors.R

setMethod("pvalues", signature(r="OBOHyperGResult"),
          function(r) orderedAttr(r, "pvalue"))

setMethod("oddsRatios", signature(r="OBOHyperGResult"),
          function(r) orderedAttr(r, "oddsRatio"))

setMethod("expectedCounts", signature(r="OBOHyperGResult"),
          function(r) orderedAttr(r, "expCount"))

## the function entrezGeneIdUniverse() is defined
## in GOHyperGResults-accessors.R

setMethod("geneIdUniverse", signature(r="OBOHyperGResult"),
          function(r, cond=TRUE) {
            if (cond && conditional(r))
              nodeData(r@goDag, n=nodes(r@goDag)[r@pvalue.order],
                       attr="condGeneIds")
            else
              entrezGeneUniverse(r)
          })

setMethod("summary", signature(object="OBOHyperGResult"),
          function(object, pvalue=pvalueCutoff(object),
                   categorySize=NULL) {
            df <- callNextMethod(object=object, pvalue=pvalue,
                                 categorySize=categorySize)
            if (nrow(df) == 0) {
              df$Term <- character(0)
              return(df)
            }
            oboIds <- df[[1]]
            df$Term <- object@gscDescriptions[oboIds]
            df
          })

setMethod("goDag", signature(r="GOHyperGResult"),
          function(r) r@goDag)

orderedAttr <- function(r, attr) {
    unlist(nodeData(r@goDag, attr=attr))[r@pvalue.order]
}

setMethod("pvalues", signature(r="GOHyperGResult"),
          function(r) orderedAttr(r, "pvalue"))

setMethod("oddsRatios", signature(r="GOHyperGResult"),
          function(r) orderedAttr(r, "oddsRatio"))

setMethod("expectedCounts", signature(r="GOHyperGResult"),
          function(r) orderedAttr(r, "expCount"))


entrezGeneUniverse <- function(r) {
    nodeData(r@goDag, n=nodes(r@goDag)[r@pvalue.order],
             attr="geneIds")
}

setMethod("geneIdUniverse", signature(r="GOHyperGResult"),
          function(r, cond=TRUE) {
              if (cond && conditional(r))
                nodeData(r@goDag, n=nodes(r@goDag)[r@pvalue.order],
                         attr="condGeneIds")
              else
                entrezGeneUniverse(r)
          })


inducedTermGraph <- function(r, id, children=TRUE, parents=TRUE) {
    if (!children && !parents)
      stop("children and parents can't both be FALSE")
    ## XXX: should use more structure here
    goOnt <- testName(r)[2]
    goKidsEnv <- GOenv(paste(goOnt, "CHILDREN", sep=""))
    goParentsEnv <- GOenv(paste(goOnt, "PARENTS", sep=""))
    goIds <- character(0)

    wantedNodes <- id
    ## children
    if (children) {
        wantedNodes <- c(wantedNodes,
                         unlist(edges(goDag(r))[id], use.names=FALSE))
    }
    ## parents
    g <- reverseEdgeDirections(goDag(r))
    if (parents) {
        wantedNodes <- c(wantedNodes,
                         unlist(edges(g)[id], use.names=FALSE))
    }
    wantedNodes <- unique(wantedNodes)
    g <- subGraph(wantedNodes, g)

    ## expand; add children and/or parents that are not present in g,
    ## but are definedin the GO data.
    if (children) {
        for (goid in id) {
            kids <- unique(goKidsEnv[[goid]])
            for (k in kids) {
                if (is.na(k)) next
                if (!(k %in% nodes(g))) {
                    g <- addNode(k, g)
                    g <- addEdge(k, goid, g)
                }
            }
        }
    }
    if (parents) {
        for (goid in id) {
            elders <- unique(goParentsEnv[[goid]])
            for (p in elders) {
                if (is.na(p)) next
                if (!(p %in% nodes(g))) {
                    g <- addNode(p, g)
                    g <- addEdge(goid, p, g)
                }
            }
        }
    }
    g
}


## FIXME: perhpas it doesn't make sense to exclude the untestable GO terms.
## maybe it would be better to keep them in as it will be less confusing?
plotGOTermGraph <- function(g, r=NULL, add.counts=TRUE,
                            max.nchar=20,
                            node.colors=c(sig="lightgray", not="white"),
                            node.shape="plaintext",
                            ...) {
    requireNamespace("Rgraphviz") || stop("'Rgraphviz' package needed")
    makeNodeAttrs <- Rgraphviz::makeNodeAttrs
    n <- nodes(g)
    termLab <-
      if ("term" %in% names(nodeDataDefaults(g))) {
          unlist(nodeData(g, attr="term"))
      } else n

    if (!is.null(max.nchar))
      termLab <- sapply(termLab, substr, 1L, max.nchar, USE.NAMES=FALSE)
    ncolors <- rep(node.colors["not"], length(n))
    if (!is.null(r) && add.counts) {
        if (is.null(names(node.colors)) ||
            !all(c("sig", "not") %in% names(node.colors)))
          stop(paste("invalid node.colors arg:",
                     "must have named elements 'sig' and 'not'"))
        resultTerms <- names(pvalues(r))
        ncolors <- ifelse(n %in% sigCategories(r), node.colors["sig"],
                          node.colors["not"])
        counts <- sapply(n, function(x) {
            if (x %in% resultTerms) {
                paste(geneCounts(r)[x], "/",
                      universeCounts(r)[x],
                      sep="")
            } else {
                "0/??"
            }
        })
        nlab <- paste(termLab, counts)
    } else {
        nlab <- termLab
    }
    nattr <- makeNodeAttrs(g,
                           label=nlab,
                           shape=node.shape,
                           fillcolor=ncolors,
                           fixedsize=FALSE)
    plot(g, ..., nodeAttrs=nattr)
}


termGraphs <- function(r, id=NULL, pvalue=NULL, use.terms=TRUE) {
    if (!is.null(id) && !is.null(pvalue))
      warning("ignoring pvalue arg since GO IDs where specified")
    if (missing(pvalue) || is.null(pvalue))
      pvalue <- pvalueCutoff(r)
    if (is.null(id))
      goids <- sigCategories(r, pvalue)
    else
      goids <- id
    g <- subGraph(goids, goDag(r))
    if (use.terms) {
        nodeDataDefaults(g) <- list(term=as.character(NA))
        nodeData(g, attr="term") <-
          as.character(sapply(mget(nodes(g), GOTERM), Term))
    }
    g <-  reverseEdgeDirections(g)
    cc <- connectedComp(g)
    sapply(cc, subGraph, g)
}

.onAttach <- function(libname, pkgname) {
    sillyName <- require
    if (.Platform$OS.type == "windows" && sillyName("Biobase")
        && interactive() && .Platform$GUI == "Rgui") {
        addVigs2WinMenu("GOstats")
    }
}

GOenv <- function(what) {
    getAnnMap(what, "GO", load=TRUE,
              type=c("db", "env"))
}

.get_eg_to_go_fun <- function(mapfun, chip, reverse=FALSE) {
    if (!is.null(chip) && is.character(chip)) {
        if (!is.null(mapfun) && is.function(mapfun))
          warning("ignoring 'chip' argument in favor of 'mapfun'")
        else {
            ## create mapfun from chip
            mapfun <- tryCatch({
                eg2go <- make_eg_to_go_map(chip)
                if (reverse)
                  eg2go <- revmap(eg2go)
                function(x) mget(x, eg2go, ifnotfound=NA)
            }, error=function(e) {
                ## if only we had classed exceptions!  and we can't
                ## reliable grep the condition message since it may be
                ## localized.
                msg <- paste(conditionMessage(e),
                             "\nDB-based version of ", chip, " not found.",
                             "\nReverting to use of environment-based GO")
                warning(msg, call.=FALSE)
                NULL
            })
        }
    }

	if (!is.function(mapfun)) {
		stop("either mapfun or chip must be specified")
	}

	mapfun
}

## Given a set of Entrez IDs obtain the GO graph that has all
## GO ids that those genes are annotated at, and
makeGOGraph <- function (x, Ontology="MF", removeRoot=TRUE,
                         mapfun=NULL, chip=NULL)
{
    eg2gofun <- .get_eg_to_go_fun(mapfun, chip)
    match.arg(Ontology, c("MF", "BP", "CC"))
    wh <- paste(Ontology, "PARENTS", sep = "")
    dataenv = GOenv(wh)
    newNodes <- eg2gofun(x)
    if( length(newNodes) == 1)
       bd = is.na(newNodes[[1]])
    else
       bd = is.na(newNodes)
    newNodes <- newNodes[!bd]

    newNodes <- lapply(newNodes, function(x) x[sapply(x,
                         function(x) {if (is.na(x$Ontology) )
                                          return(FALSE)
                                      else
                                          x$Ontology == Ontology})])
    oldEdges <- vector("list", length = 0)
    oldNodes <- vector("character", length = 0)
    for (i in 1:length(newNodes)) {
        newN <- newNodes[[i]]
        if (!length(newN))
          next
        newN <- unique(subListExtract(newN, "GOID", simplify=TRUE))
        done <- FALSE
        while (!done) {
            newN <- newN[!(newN %in% oldNodes)]
            if (length(newN) == 0)
                done <- TRUE
            else {
                oldNodes <- c(oldNodes, newN)
                numE <- length(newN)
                nedges <- mget(newN, envir = dataenv, ifnotfound=NA)
                nedges <- nedges[!is.na(nedges)]
                oldEdges <- c(oldEdges, nedges)
                if (length(nedges) > 0)
                  newN <- sort(unique(unlist(nedges)))
                else newN <- NULL
            }
        }
    }
    rE <- vector("list", length = length(oldNodes))
    names(rE) <- oldNodes
    rE[names(oldEdges)] <- oldEdges
    rE <- lapply(rE, function(x) x <- which(oldNodes %in% x))
    names(rE) <- oldNodes

    rval = new("graphNEL", nodes = oldNodes, edgeL = lapply(rE,
        function(x) list(edges = x)), edgemode = "directed")

    if (removeRoot) {
        ## For compatibility with old GO packages, check for "GO:0003673" if
        ## "all" is not found.
        if (is.element("all", nodes(rval)))
          rval = removeNode("all", rval)
        else if (is.element("GO:0003673", nodes(rval)))
          rval = removeNode("GO:0003673", rval)
    }
    rval
}


##start with one specific term and find it's more general terms
## if A has edges to B, C and D
##then at step 2, the nodes are A,B,C,D; new nodes B,C,D
##we need to find edges for each of them
##we're going to build a graphNEL
GOGraph = function(x, dataenv) {
    ##this is the old oneGOGraph code - but it just works for
    ##multiple inputs
    oldEdges <- vector("list", length=0)
    oldNodes <- vector("character", length=0)
    newN <- x
    done <- FALSE
    while( !done ) {
        newN <- newN[!(newN %in% oldNodes)]
        if( length(newN) == 0 )
            done <- TRUE
        else {
            oldNodes <- c(oldNodes, newN)
            numE <- length(newN)
            nedges <- mget(newN, envir=dataenv, ifnotfound=NA)
            nedges <- nedges[!is.na(nedges)]
            oldEdges <- c(oldEdges, nedges)
            if( length(nedges) > 0 )
                newN <- sort(unique(unlist(nedges)))
            else
                newN <- NULL
        }
    }
    rE <- vector("list", length=length(oldNodes))
    names(rE) <- oldNodes
    ##DON'T CHANGE: we must have the indices in the edgeL slot be
    ##   integer offsets  to the node list
    rE[names(oldEdges)] <- oldEdges
    rE <- lapply(rE, function(x) match(x, oldNodes))
    names(oldNodes) = oldNodes
    return(new("graphNEL", nodes=oldNodes, edgeL = lapply(rE,
               function(x) list(edges=x)),edgemode="directed" ))
}

##to be deprecated
oneGOGraph <- function(x, dataenv) {
    if( length(x) > 1 )
        stop("wrong number of GO terms")
    GOGraph(x, dataenv)
}

 ##similarity functions - based on graphs
 simUI = function(g1, g2) {
     if(!is(g1, "graph") || !is(g2, "graph") )
         stop("only works for graphs")
     n1 = nodes(g1); n2 = nodes(g2)
    length(intersect(n1, n2))/length(union(n1, n2))
 }

 simLP = function(g1, g2) {
    if(!is(g1, "graph") || !is(g2, "graph") )
         stop("only works for graphs")
    ig <- intersection(g1, g2)
    if( numNodes(ig) < 2 ) return(0)
    lfi <- leaves(ig, "in")
    degs = degree(ig)
    root = names(degs$outDegree)[degs$outDegree == 0]
    paths = sp.between(ig, lfi, root)
    plens = subListExtract(paths, "length", simplify=TRUE)
    max(plens)
}

simLL = function(ll1, ll2, Ontology="MF", measure="LP", dropCodes=NULL,
                 mapfun=NULL, chip=NULL) {
    eg2gofun <- .get_eg_to_go_fun(mapfun, chip)
     ## helper function to get the right GOIDs
     .getWHEC = function(llid, wh, eCodes) {
         x = eg2gofun(llid)[[1L]]
         if( !is.null(eCodes) )
           x = dropECode(x, eCodes)
         unique(unlist(getOntology(x, wh)))
     }

    wh = match.arg(Ontology, c("MF", "BP", "CC"))
    ll1GO = .getWHEC(ll1, wh, dropCodes)
    ll2GO = .getWHEC(ll2, wh, dropCodes)
    dataenv = GOenv(paste(wh, "PARENTS", sep=""))
    g1 = GOGraph(ll1GO, dataenv)
    g2 = GOGraph(ll2GO, dataenv)
    if (length(nodes(g1)) == 0 || length(nodes(g2)) == 0)
      return(NA)
    sm = match.arg(measure, c("LP", "UI"))
    sim = switch(sm,
           LP = simLP(g1, g2),
           UI = simUI(g1, g2))
    return(list(sim=sim, measure=measure, g1=g1, g2=g2))
  }

## ----Setup, echo=TRUE, message=FALSE------------------------------------------
library("ALL")
library("hgu95av2.db")
library("GO.db") 
library("annotate")
library("genefilter") 
library("GOstats") 
library("RColorBrewer")
library("xtable") 
library("Rgraphviz")

## ----universeSizeVsPvalue, eval=TRUE, echo=FALSE, results='hide'--------------
hg_tester <-function(size){
  numFound <- 10 
  numDrawn <- 400 
  numAtCat <- 40
  numNotAtCat <- size - numAtCat 
  phyper(numFound-1, numAtCat,numNotAtCat, numDrawn, lower.tail=FALSE)
}
pv1000 <- hg_tester(1000)
pv5000 <- hg_tester(5000)

## ----bcrAblOrNegSubset, results='hide', echo=TRUE, cache=TRUE-----------------
data(ALL, package="ALL") 
## For this data we can have ALL1/AF4 or BCR/ABL 
subsetType <- "ALL1/AF4"
## Subset of interest: 37BRC/ABL + 42NEG = 79 samples 
Bcell <- grep("^B", as.character(ALL$BT))
bcrAblOrNegIdx <- which(as.character(ALL$mol) %in% c("NEG", subsetType))
bcrAblOrNeg <- ALL[, intersect(Bcell, bcrAblOrNegIdx)]
bcrAblOrNeg$mol.biol = factor(bcrAblOrNeg$mol.biol)

## ----nsFiltering-noEntrez, results='hide', echo=TRUE, cache=TRUE--------------
##Remove genes that have no entrezGene id 
entrezIds <- mget(featureNames(bcrAblOrNeg), envir=hgu95av2ENTREZID) 
haveEntrezId <- names(entrezIds)[sapply(entrezIds, function(x) !is.na(x))]
numNoEntrezId <- length(featureNames(bcrAblOrNeg)) - length(haveEntrezId) 
bcrAblOrNeg <- bcrAblOrNeg[haveEntrezId, ]

## ----nsFiltering-noGO, results='hide', echo=TRUE, cache=FALSE-----------------
## Remove genes with no GO mapping 
haveGo <- sapply(mget(featureNames(bcrAblOrNeg), hgu95av2GO), 
                 function(x){
                   if (length(x) == 1 && is.na(x)) 
                     FALSE 
                   else TRUE 
                   })
numNoGO <- sum(!haveGo)
bcrAblOrNeg <- bcrAblOrNeg[haveGo, ]

## ----nsFiltering-IQR, results='hide', echo=TRUE, cache=TRUE-------------------
## Non-specific filtering based on IQR 
iqrCutoff <- 0.5 
bcrAblOrNegIqr <- apply(exprs(bcrAblOrNeg), 1, IQR) 
selected <- bcrAblOrNegIqr > iqrCutoff

## Drop those that are on the Y chromosome 
## because there is an imbalance of men and women by group 
chrN <- mget(featureNames(bcrAblOrNeg), envir=hgu95av2CHR) 
onY <- sapply(chrN, function(x) any(x=="Y"))
onY[is.na(onY)] <- FALSE
selected <- selected & !onY

nsFiltered <- bcrAblOrNeg[selected, ]

## ----nsFiltering-unique, results='hide', echo=TRUE, cache=TRUE----------------
numNsWithDups <- length(featureNames(nsFiltered))
nsFilteredIqr <- bcrAblOrNegIqr[selected] 
uniqGenes <- findLargest(featureNames(nsFiltered), nsFilteredIqr, 
                         "hgu95av2")
nsFiltered <- nsFiltered[uniqGenes, ] 
numSelected <- length(featureNames(nsFiltered))

##set up some colors 
BCRcols = ifelse(nsFiltered$mol == subsetType, "goldenrod", "skyblue")
cols = brewer.pal(10, "RdBu")

## ----defineGeneUniverse, echo=TRUE, results='hide', cache=TRUE----------------
## Define gene universe based on results of non-specific filtering
affyUniverse <- featureNames(nsFiltered)
entrezUniverse <- unlist(mget(affyUniverse, hgu95av2ENTREZID))
if (any(duplicated(entrezUniverse)))
  stop("error in gene universe: can't have duplicate Entrez Gene Ids")

## Also define an alternate universe based on the entire chip
chipAffyUniverse <- featureNames(bcrAblOrNeg)
chipEntrezUniverse <- mget(chipAffyUniverse, hgu95av2ENTREZID)
chipEntrezUniverse <- unique(unlist(chipEntrezUniverse)) 

## ----parametric1, echo=TRUE, results='hide', cache=TRUE-----------------------
ttestCutoff <- 0.05
ttests = rowttests(nsFiltered, "mol.biol")

smPV = ttests$p.value < ttestCutoff

pvalFiltered <- nsFiltered[smPV, ] 
selectedEntrezIds <- unlist(mget(featureNames(pvalFiltered), 
                                 hgu95av2ENTREZID))

## ----eval=FALSE---------------------------------------------------------------
# resultList <- lapply(lisOfParamObjs, hyperGTest)

## ----withYourData1, eval=FALSE------------------------------------------------
# entrezUniverse <- unlist(mget(featureNames(yourData),
#                               hgu95av2ENTREZID))
# if (any(duplicated(entrezUniverse)))
#  stop(\"error in gene universe: can't have duplicate Entrez Gene Ids")
# 
# pvalFiltered <- yourData[hasSmallPvalue, ]
# selectedEntrezIds <-unlist(mget(featureNames(pvalFiltered),
#                                 hgu95av2ENTREZID))

## ----standardHyperGeo, echo=TRUE, results='hide', cache=TRUE------------------
hgCutoff<- 0.001 
params <- new("GOHyperGParams", 
              geneIds=selectedEntrezIds,
              universeGeneIds=entrezUniverse, 
              annotation="hgu95av2.db",
              ontology="BP", 
              pvalueCutoff=hgCutoff, 
              conditional=FALSE,
              testDirection="over")

## ----condHyperGeo, echo=TRUE, results='hide', cache=TRUE----------------------
paramsCond <- params 
conditional(paramsCond) <- TRUE 

## ----standardHGTEST, cache=TRUE, echo=TRUE, results='hide'--------------------
hgOver <- hyperGTest(params)
hgCondOver <- hyperGTest(paramsCond)

## ----HGTestAns----------------------------------------------------------------
hgOver 
hgCondOver

## ----summaryEx----------------------------------------------------------------
df <- summary(hgOver)
names(df) # the columns 
dim(summary(hgOver, pvalue=0.1))
dim(summary(hgOver, categorySize=10)) 

## ----resultAccessors----------------------------------------------------------
pvalues(hgOver)[1:3]

oddsRatios(hgOver)[1:3]

expectedCounts(hgOver)[1:3]

geneCounts(hgOver)[1:3]
universeCounts(hgOver)[1:3]

length(geneIds(hgOver)) 
length(geneIdUniverse(hgOver)[[3]])

## GOHyperGResult _only_ 
## (NB: edges go from parent to child)
goDag(hgOver)

geneMappedCount(hgOver) 
universeMappedCount(hgOver)

conditional(hgOver) 
testDirection(hgOver)
testName(hgOver)

## ----htmlReportExample, results='hide', echo=TRUE-----------------------------
htmlReport(hgCondOver, file="ALL_hgco.html")

## ----helperFunc, echo=FALSE, results='hide', cache=TRUE-----------------------
sigCategories <- function(res, p){
  if (missing(p)) 
    p <- pvalueCutoff(res) 
  pv <- pvalues(res) 
  goIds <- names(pv[pv < p])
  goIds 
}

## ----plotFuns, echo=FALSE, results='hide', cache=TRUE-------------------------
coloredGoPlot <- function(ccMaxGraph, hgOver, hgCondOver) {
  nodeColors <- sapply(nodes(ccMaxGraph),
                       function(n) {
                         if (n %in% sigCategories(hgCondOver))
                             "dark red"
                          else if (n %in% sigCategories(hgOver))
                            "pink" 
                          else 
                            "gray"
                        }) 
 nattr <- makeNodeAttrs(ccMaxGraph,
                        label=nodes(ccMaxGraph), 
                        shape="ellipse", 
                        fillcolor=nodeColors,
                        fixedsize=FALSE)
 plot(ccMaxGraph, nodeAttrs=nattr)
}

getMaxConnCompGraph <- function(hgOver, hgCondOver){
  ##uGoDagRev \<-ugraph(goDag(hgOver)) 
  sigNodes <- sigCategories(hgOver) 
  ##adjNodes \<- unlist(adj(uGoDagRev, sigNodes))
  adjNodes <- unlist(adj(goDag(hgOver),sigNodes))
  displayNodes <- unique(c(sigNodes, adjNodes)) 
  displayGraph <- subGraph(displayNodes, goDag(hgOver))
  cc <- connComp(displayGraph)
  ccSizes <- listLen(cc) 
  ccMaxIdx <- which(ccSizes == max(ccSizes))
  ccMaxGraph <- subGraph(cc[[ccMaxIdx]], displayGraph)
  ccMaxGraph
}


## ----info---------------------------------------------------------------------
sessionInfo()

##########################################
##
##  Copyright 2005 James W. MacDonald 
##
##  A function to output a table from running
##  GOHyperG() on a set of Affymetrix probe IDs
##
##
##  Modified 6-9-05 to output the table
##  Added hyperG2Affy 10-10-2005
##   - This function takes GOIDs form hyperGtable and outputs
##     a list giving the Affy IDs associated with each GOID
##
##
##  Moved hyperGtable and hyperG2Affy to GOstats 2-23-06
##
##  Added universe argument 3-22-06
##
## Made hyperGtable and hyperG3Affy defunct 10-2-07
###########################################

probeSetSummary <- function(result, pvalue, categorySize, sigProbesets,
                            ids = "ENTREZID") {
    if (!is(result, "GOHyperGResult"))
      stop("result must be a GOHyperGResult instance (or subclass)")
    ## build reverse map
    ps2egEnv <- getAnnMap(ids, annotation(result))
    psids <- ls(ps2egEnv)
    ## each psid maps to _exactly_ one egid
    ## Need to remove NAs.  This is where the database stuff
    ## will be _much_ easier.
    havePsids <- eapply(ps2egEnv, function(ids) {
        if (length(ids) == 1 && is.na(ids))
          FALSE
        else
          TRUE
    })
    ord <- order(names(havePsids))
    psids <- psids[unlist(havePsids)[ord]]
    eg2ps <- split(psids, unlist(mget(psids, ps2egEnv)))

    if (missing(pvalue))
      pvalue <- pvalueCutoff(result)
    if (missing(categorySize))
      categorySize <- NULL
    ## XXX FIXME:
    summary <- Category:::XXX_getSummaryGeneric_XXX()
    goids <- summary(result, pvalue, categorySize)[,1]
    sigegids <- geneIds(result)
    egids <- geneIdUniverse(result)[goids]
    psetids <- lapply(egids, function(ids) {
        ids <- as.character(ids)
        have <- ids %in% sigegids
        ans <- eg2ps[ids[have]]
        if (length(ans))
          unlist(ans)
        else
          NULL
    })
    psetidsNULL <- sapply(psetids, is.null)
    psetids <- psetids[!psetidsNULL]
    if(missing(sigProbesets)){
        selectedProbeSetIDs <- names(geneIds(result))
        if(is.null(selectedProbeSetIDs))
            warning(paste("The vector of geneIds used to create the GOHyperGParams",
                          "object was not a named vector.\nIf you want to know the",
                          "probesets that contributed to this result\neither use",
                          "a named vector for geneIds, or pass a vector of probeset IDs\n",
                          "via sigProbesets.\n", sep=""),
                    call.=FALSE)
    }else{
        selectedProbeSetIDs <- sigProbesets
    }
    selectedInd <- lapply(psetids, function(ids) {
        ids %in% selectedProbeSetIDs
    })
    egids <- lapply(psetids, function(ids) unlist(mget(ids, ps2egEnv)))
    out <- vector(mode="list", length=length(psetids))
    for (i in seq(along=psetids)) {
        out[[i]] <- data.frame(EntrezID=egids[[i]],
                               ProbeSetID=psetids[[i]],
                               selected=as.integer(selectedInd[[i]]),
                               row.names=NULL,
                               stringsAsFactors=FALSE)
    }
    names(out) <- names(psetids)
    out
}


setMethod("summary", signature(object="GOHyperGResult"),
          function(object, pvalue=pvalueCutoff(object),
                   categorySize=NULL, htmlLinks=FALSE) {
              AMIGO_URL <- "http://amigo.geneontology.org/amigo/term/%s"
              df <- callNextMethod(object=object, pvalue=pvalue,
                                   categorySize=categorySize)
              if (nrow(df) == 0)  {
                  df$Term <- character(0)
                  return(df)
              }
              goIds <- df[[1]]
              goTerms <- sapply(mget(goIds, GOenv("TERM")), Term)
              if (htmlLinks) {
                  goIdUrls <- sapply(goIds,
                                     function(x) sprintf(AMIGO_URL, x))
                  goTerms <- paste('<a href="', goIdUrls, '">', goTerms,
                                   '</a>', sep="")
              }
              df$Term <- goTerms
              df
          })

setMethod("summary", signature(object="OBOHyperGResult"),
    function(object, pvalue=pvalueCutoff(object), categorySize=NULL)
{
    df <- callNextMethod(object=object, pvalue=pvalue,
                         categorySize=categorySize)
    if (nrow(df) == 0) {
        df$Term <- character(0)
        return(df)
    }
    oboIds <- df[[1]]
    df$Term <- object@gscDescriptions[oboIds]
    df
})

setMethod("htmlReport", signature(r="GOHyperGResult"),
          function(r, file="", append=FALSE, label="",
                   digits=3, summary.args=list(htmlLinks=TRUE))
          {
              ## We define a method here to pass htmlLinks=TRUE via
              ## ... as the default.

              ## NB: it "should" be enough to call callNextMethod() with
              ## no arguments, but it doesn't work.
              callNextMethod(r=r, file=file, append=append, label=label,
                             digits=digits, summary.args=summary.args)
          })

## ----available Schemas, message=FALSE, warning=FALSE--------------------------
library("AnnotationForge")
available.dbschemas()

## ----Acquire annotation data, message=FALSE-----------------------------------
library("org.Hs.eg.db")
frame = toTable(org.Hs.egGO)
goframeData = data.frame(frame$go_id, frame$Evidence, frame$gene_id)
head(goframeData)

## ----transformGOFrame, message=FALSE, warning=FALSE---------------------------
goFrame=GOFrame(goframeData,organism="Homo sapiens")
goAllFrame=GOAllFrame(goFrame)

## ----Make GSC, message=FALSE--------------------------------------------------
library("GSEABase")
gsc <- GeneSetCollection(goAllFrame, setType = GOCollection())

## ----make parameter, message=FALSE--------------------------------------------
library("GOstats")
universe = Lkeys(org.Hs.egGO)
genes = universe[1:500]
params <- GSEAGOHyperGParams(name="My Custom GSEA based annot Params", 
                             geneSetCollection=gsc, 
                             geneIds = genes, 
                             universeGeneIds = universe, 
                             ontology = "MF", 
                             pvalueCutoff = 0.05, 
                             conditional = FALSE, 
                             testDirection = "over")

## ----call HyperGTest----------------------------------------------------------
Over <- hyperGTest(params)
head(summary(Over))

## ----KEGGFrame object---------------------------------------------------------
frame = toTable(org.Hs.egPATH)
keggframeData = data.frame(frame$path_id, frame$gene_id)
head(keggframeData)
keggFrame=KEGGFrame(keggframeData,organism="Homo sapiens")

## ----KEGG Parameters----------------------------------------------------------
gsc <- GeneSetCollection(keggFrame, setType = KEGGCollection())
universe = Lkeys(org.Hs.egGO)
genes = universe[1:500]
kparams <- GSEAKEGGHyperGParams(name="My Custom GSEA based annot Params", 
                               geneSetCollection=gsc, 
                               geneIds = genes, 
                               universeGeneIds = universe,  
                               pvalueCutoff = 0.05, 
                               testDirection = "over")
kOver <- hyperGTest(params)
head(summary(kOver))

## ----info---------------------------------------------------------------------
sessionInfo()

##FIRST LET'S SEE WHAT THE P-VALUE PROBLEM LOOKS LIKE

##FIXME: just ran the first simulation set1 - need to do set2 again
##and start looking at the appropriate set of adjustments
 library("GOstats")
 library("hgu95av2.db")

 w1 <- as.list(hgu95av2LOCUSID)

 w2 <- unique(unlist(w1))

 set.seed(123)

 myLL <- sample(w2, 100)

 xx <- hyperGTest(myLL)

 xx$numLL

 xx$numInt
[1] 60

 sum(xx$pvalues < 0.01)
[1] 9

 rval = NULL
 for(i in 1:10) {
     mx = sample(w2, 100)
     xx = hyperGTest(mx)
     rval[[i]] = list(nI = xx$numInt,gC= xx$goCounts,pV= xx$pvalues)
 }

 nI = sapply(rval, function(x) x$nI)
 gC = sapply(rval, function(x) length(x$gC))

 countpvs = function(il, pv=0.05)
    sapply(il, function(x) sum(x$pV < pv))

 nPV = countpvs(rval)

 sapply(rval, function(x) max(x$pV))


##analysis

 analyzeRun = function(x, pv=0.05) {
     nI = sapply(x, function(y) y$nI)
     gC = sapply(x, function(y) length(y$gC))
     nPV = countpvs(x, pv)
     return(list(nI=nI, gC=gC, pvs = nPV))
 }

##for set1
set1 = genPVs(100,100)

s1a = analyzeRun(set1)
mean(s1a$pvs/s1a$gC) #should be about 0.05, but it isn't

s2a = analyzeRun(set1, 0.01)
mean(s2a$pvs/s2a$gC)  ##is pretty close to 0.01

s3a = analyzeRun(set1, 0.1)
mean(s3a$pvs/s3a$gC)  ##about 3 times what it should be

#for set2
set2 = genPVs(100,100)

s2.05 = analyzeRun(set2)
mean(s2.05$pvs/s2.05$gC) #should be about 0.05, but it isn't

s2.01 = analyzeRun(set2, 0.01)
mean(s2.01$pvs/s2.01$gC)  ##is pretty close to 0.01

s2.1 = analyzeRun(set2, 0.1)
mean(s2.1$pvs/s2.1$gC)  ##about 3 times what it should be

##look at int g size

set3 = genPVs(500, 75)


##run a simulation

 genPVs = function(nsim, nsamp, verbose=TRUE) {
      rval = NULL
      for(i in 1:nsim) {
          mx = sample(w2, nsamp)
          rval[[i]] = hyperGTest(mx)
          if( verbose )
              print(paste("finished", i, "of", nsim))
      }
      return(rval)
  }

 set.seed(455)

 set1 = genPVs(100, 100)


#################
## compare dists
#################

 data(Ndists)
 data(Bdists)

 bothBad = (Ndists==Inf & Bdists==Inf)

 badN = (Ndists==Inf & Bdists !=Inf)

 badB = (Bdists==Inf & Ndists !=Inf)

setClass("GOHyperGResult",
         contains="HyperGResultBase",
         representation=representation(
           goDag="graph",
           pvalue.order="integer",
           conditional="logical"),
         prototype=prototype(
           testName="GO",
           pvalueCutoff=0.01,
           goDag=new("graphNEL")))

## this is for OBO
setClass("OBOHyperGResult",
         contains="HyperGResultBase",
         representation=representation(
           goDag="graph",
           gscDescriptions="character",
           pvalue.order="integer",
           conditional="logical"),
         prototype=prototype(
           testName="GO",
           pvalueCutoff=0.01,
           goDag=new("graphNEL")))

setMethod("hyperGTest", "GOHyperGParams", function(p) {
    .hyperGTestInternal(p, "GOHyperGResult", getGoToChildGraph)
})

setMethod("hyperGTest", "OBOHyperGParams", function(p) {
    .hyperGTestInternal(p, "OBOHyperGResult", getGoToChildOBOgraph)
})

.hyperGTestInternal <- function(p, resultsClassName, childGraphFun) {
  p <- makeValidParams(p)
  p@universeGeneIds <- universeBuilder(p)
  ## preserve names on geneIds
  p@geneIds <- p@geneIds[p@geneIds %in% p@universeGeneIds]
  ##FIXME: since the unique values in this list are the
  ## the universe, it might be nice to have that somewhere
  cat2Entrez <- categoryToEntrezBuilder(p)
  ## build the GO graph for the relevant GO nodes
  goIds <- names(cat2Entrez)
  goDag <- childGraphFun(p, goIds)
  nodeDataDefaults(goDag, "pvalue") <- 1
  nodeDataDefaults(goDag, "geneIds") <- numeric(0)
  nodeDataDefaults(goDag, "condGeneIds") <- numeric(0)
  nodeDataDefaults(goDag, "oddsRatio") <- 1
  nodeDataDefaults(goDag, "expCount") <- numeric(0)
  ## store the Entrez Gene IDs as attrs on the GO DAG.
  nodeData(goDag, n=names(cat2Entrez),
           attr="geneIds") <- cat2Entrez
  ## now iterate leaves first doing tests and conditioning
  ## on all significant children.
  ## FIXME: consider replacing with RBGL tsort?
  needsProc <- goDag
  complete <- character(0)
  SIGNIF <- p@pvalueCutoff
  SIGODD <- p@orCutoff
  MINSIGSZE <- p@minSizeCutoff
  MAXSIGSZE <- p@maxSizeCutoff
  while (length(nodes(needsProc))) {
    numKids <- sapply(edges(needsProc), length)
    noKids <- names(numKids[numKids == 0])
    curCat2Entrez <- cat2Entrez[noKids]
    if (p@conditional) {
      curCatKids <- edges(goDag)[names(curCat2Entrez)]
      curCatKids <- removeLengthZero(curCatKids)
      if (length(curCatKids)) { ## sanity check
        ## they should be all complete
        stopifnot(all(unlist(curCatKids) %in% complete))
      }
      curCat2Entrez <- removeSigKidGenes(curCatKids, goDag,
                                         curCat2Entrez,
                                         SIGNIF, SIGODD, MINSIGSZE, MAXSIGSZE,
                                         cat2Entrez)
      ## Store the conditioned cat => entrez map
      nodeData(goDag, n=names(curCat2Entrez),
               attr="condGeneIds") <- curCat2Entrez
    }
    stats <- .doHyperGTest(p, curCat2Entrez, cat2Entrez,
                           p@geneIds)
    
    ## store the pvals, mark these nodes as complete,
    ## then compute the next set of nodes to do.  
    noKids <- names(curCat2Entrez)
    ## drop names on pvals to avoid weird names upon unlisting
    nodeData(goDag, n=noKids,
             attr="pvalue") <- as.numeric(stats$p)
    nodeData(goDag, n=noKids,
             attr="oddsRatio") <- as.numeric(stats$odds)
    nodeData(goDag, n=noKids,
             attr="expCount") <- as.numeric(stats$expected)
    complete <- c(complete, noKids)
    hasKids <- names(numKids[numKids > 0])
    needsProc <- subGraph(hasKids, needsProc)
  } ## end while
  pvals <- unlist(nodeData(goDag, attr="pvalue"))
  ##modded for future generic GenSet baseed objects...
  if(class(p@datPkg)!="GeneSetCollectionDatPkg"){
    annotation = annotation(p)
  }else if(class(p@datPkg)=="GeneSetCollectionDatPkg"){
    annotation = "Based on a GeneSetCollection Object"
  }
  objparams <- list(Class=resultsClassName,
                    goDag=goDag,
                    annotation=annotation,
                    geneIds=p@geneIds,
                    testName=categoryName(p),
                    testDirection=p@testDirection,
                    pvalueCutoff=p@pvalueCutoff,
                    pvalue.order=order(pvals),
                    conditional=p@conditional)
  if (resultsClassName == "OBOHyperGResult") {
      desc <- sapply(p@datPkg@geneSetCollection[nodes(goDag)], description)
      objparams$gscDescriptions <- setNames(desc, nodes(goDag))
  }
  do.call("new", objparams)
}





removeLengthZero <- function(x) {
    wanted <- sapply(x, function(z) length(z) > 0)
    x[wanted]
}


getGoToChildGraph <- function(p, goIds) {
    ## We might want a method here so that we can dispatch on
    ## a SQLite-based GO package.
    goEnv <- GOenv(paste(p@ontology, "CHILDREN", sep=""))
    gobpkids <- eapply(goEnv, function(x) {
        intersect(x, goIds)
    })
    gobpkids <- list2env(gobpkids)
    GOGraph(goIds, gobpkids)
}

## this is for OBO
getGoToChildOBOgraph <- function(p, goIds) {
  childrenEdges <- edges(as(t(as(p@datPkg@oboGraph, "matrix")), "graphNEL"))
  childrenEdges <- lapply(childrenEdges, intersect, goIds)
  childrenEdges <- list2env(childrenEdges)
  GOGraph(goIds, childrenEdges)
}


## ROBERT 30.7.2017. Added arguments SIGODD, MINSIGSZE, MAXSIGSZE
## to consider odds ratio and gene set size (minimum and maximum)
## when removing children from significant terms. Thus the notion
## of a "significant" term is extended to those meeting also these
## parameters, when given.
removeSigKidGenes <- function(curCatKids, goDag, curCat2Entrez, SIGNIF,
                              SIGODD, MINSIGSZE, MAXSIGSZE, cat2Entrez) {
    if (length(curCatKids)) {
        ## keep only those kids with SIGNIF pvalue
        ## SIGODD odds ratio, MINSIGSZE minimum gene set size
        ## and MAXSIGSZE maximum gene set size
        curCatKids <- lapply(curCatKids, function(x) {
            pvKids <- nodeData(goDag, n=x, attr="pvalue")
            orKids <- nodeData(goDag, n=x, attr="oddsRatio")
            szeKids <- lengths(nodeData(goDag, n=x, attr="geneIds"))
            idx <- which(pvKids < SIGNIF & orKids >= SIGODD &
                         szeKids >= MINSIGSZE & szeKids <= MAXSIGSZE)
            if (length(idx))
              x[idx]
            else
              character(0)
        })
        curCat2EntrezCond <- list()
        for (goid in names(curCat2Entrez)) {
            ## remove entrez ids that came from 
            ## SIGNIF/SIGODD/MINSIGSZE/MAXSIGSZE children
            kids <- curCatKids[[goid]]
            if (length(kids)) {
                kidEgs <- unlist(cat2Entrez[kids])
                newEgs <- setdiff(curCat2Entrez[[goid]], kidEgs)
                ## newEgs may be length 0
                curCat2EntrezCond[[goid]] <- newEgs
            } else {
                curCat2EntrezCond[[goid]] <- curCat2Entrez[[goid]]
            }
        }
        curCat2Entrez <- curCat2EntrezCond
    }
    curCat2Entrez
}




GOHG <- function(entrezGeneIds, lib, ontology, universe=NULL,
                 conditional=FALSE, testDirection="over",
                 pvalueCutoff=0.01)
{
    if (missing(universe) || is.null(universe))
      universe <- character(0)

    if (conditional && missing(pvalueCutoff))
      stop("conditional computation requires ",
           sQuote("pvalueCutoff"), " to be specified")
    if (missing(pvalueCutoff))
      pvalueCutoff <-  0.01
    
    params <- new("GOHyperGParams",
                  geneIds=entrezGeneIds,
                  universeGeneIds=universe,
                  annotation=lib,
                  ontology=ontology,
                  testDirection=testDirection,
                  pvalueCutoff=pvalueCutoff)
    
    hyperGTest(params)
}

## ----Setup, echo=FALSE, results='hide', message=FALSE-------------------------
library("Biobase")
library("annotate")
library("GOstats")
library("xtable")
library("multtest")
library("Rgraphviz")
library("hgu95av2.db")
library("GO.db")
library("genefilter")
library("ALL")

## ----Example, results='hide', echo=FALSE--------------------------------------
## subset of interest: 37+42 samples
data(ALL)
eset <- ALL[, intersect(
    grep("^B", as.character(ALL$BT)),
    which(as.character(ALL$mol) %in% c("BCR/ABL", "NEG", "ALL1/AF4"))
)]
MB <- factor(eset$mol.bio)

## intensities above 100 in at least 7 of the samples
f1 <- kOverA(7, log2(100))
f2 <- function(x) {
    gps <- split(2^x, MB)
    mns <- sapply(gps, mean)
    if (max(mns) - min(mns) < 100) FALSE else TRUE
}
ff <- filterfun(f1, f2)
selected <- genefilter(eset, ff)
sum(selected)
esetSub <- eset[selected, ]

## ----mtFstats, results='hide', echo=FALSE-------------------------------------
pvCutOff <- 0.05

Fstats <- mt.maxT(exprs(esetSub), as.numeric(MB) - 1,
                  B = 1000, test = "f"
)
eSet <- esetSub[Fstats$index[Fstats$adjp < pvCutOff], ]
gN <- featureNames(eSet)
lls <- unlist(mget(gN, hgu95av2ENTREZID, ifnotfound = NA))
eS <- eSet[!duplicated(lls), ]
gN <- featureNames(eS)
lls <- unlist(mget(gN, hgu95av2ENTREZID, ifnotfound = NA))
lls <- as.character(lls[!is.na(lls)])
syms <- unlist(mget(gN, hgu95av2SYMBOL, ifnotfound = NA))

## ----inducedGO, echo=FALSE, results='hide'------------------------------------
gMF <- makeGOGraph(lls, "MF", chip = "hgu95av2")
gBP <- makeGOGraph(lls, "BP", chip = "hgu95av2")
gCC <- makeGOGraph(lls, "CC", chip = "hgu95av2")
nMF <- list()
lbs <- rep("", length(nodes(gMF)))
names(lbs) <- nodes(gMF)
nMF$label <- lbs
nBP <- list()
lbs <- rep("", length(nodes(gBP)))
names(lbs) <- nodes(gBP)
nBP$label <- lbs
nCC <- list()
lbs <- rep("", length(nodes(gCC)))
names(lbs) <- nodes(gCC)
nCC$label <- lbs
if (require("Rgraphviz")) {
    gMFlo <- agopen(gMF, "gMF")
    gBPlo <- agopen(gBP, "gBP")
    gCClo <- agopen(gCC, "gCC")
}

## ----GOMF, fig=TRUE, echo=FALSE, warning=FALSE, fig.cap = " The induced MF GO graph for the selected genes."----
if (require("Rgraphviz")) {
    plot(gMFlo, nodeAttrs = nMF, main = "Molecular Function")
}

## ----GOBP, fig=TRUE, echo=FALSE, warning=FALSE, fig.cap = " The induced BP GO graph for the selected genes."----
if (require("Rgraphviz")) {
    plot(gBPlo, nodeAttrs = nBP, main = "Biological Process")
}

## ----GOCC, fig=TRUE, echo=FALSE, warning=FALSE, fig.cap = "The induced CC GO graph for the selected genes."----
if (require("Rgraphviz")) {
    plot(gCClo, nodeAttrs = nCC, main = "Cellular Component")
}

## ----findhigestmeans----------------------------------------------------------
mns <- apply(exprs(eS), 1, function(x) sapply(split(x, MB), mean))
whismax <- apply(mns, 2, function(x) match(max(x), x))
maxNames <- names(mns[, 1])[whismax]
names(maxNames) <- names(whismax)
table(maxNames)

## ----genesToGO----------------------------------------------------------------
CCnodes <- nodes(gCC)
nodes2affy <- mget(CCnodes, hgu95av2GO2ALLPROBES, ifnotfound = NA)
cts <- sapply(nodes2affy, function(x) {
    wh <- names(whismax) %in% x
    table(maxNames[wh])
})
all1cts <- sapply(cts, function(x) x["ALL1/AF4"])
all1cts <- ifelse(is.na(all1cts), 0, all1cts)
## put nice names on
names(all1cts) <- names(cts)
bcrcts <- sapply(cts, function(x) x["BCR/ABL"])
bcrcts <- ifelse(is.na(bcrcts), 0, bcrcts)
names(bcrcts) <- names(cts)
negcts <- sapply(cts, function(x) x["NEG"])
negcts <- ifelse(is.na(negcts), 0, negcts)
names(negcts) <- names(cts)
ctmat <- cbind(all1cts, bcrcts, negcts)

## ----layoutandrender, eval=FALSE, warning=FALSE, results="hide"---------------
# if (require(Rgraphviz)) {
#     opar <- par(xpd = NA)
#     plotPieChart <- function(curPlot, counts, main) {
#         renderNode <- function(x) {
#             force(x)
#             y <- x * 100 + 1
#             function(node, ur, attrs = list(), radConv = 1) {
#                 nodeCenter <- getNodeCenter(node)
#                 pieGlyph(y,
#                     xpos = getX(nodeCenter),
#                     ypos = getY(nodeCenter),
#                     radius = getNodeRW(node),
#                     col = c("blue", "green", "red")
#                 )
#             }
#         }
#         drawing <- vector(mode = "list", length = nrow(counts))
#         for (i in 1:length(drawing)) {
#             drawing[[i]] <- renderNode(counts[i, ])
#         }
#         if (missing(main)) {
#             main <- "Example Pie Chart Plot"
#         }
# 
#         plot(curPlot, drawNode = drawing, main = main)
# 
#         legend(
#             x = "bottomleft", legend = c("ALL1/AF4", "BCR/ABL", "NEG"),
#             fill = c("blue", "green", "red")
#         )
#     }
#     plotPieChart(gCClo, ctmat)
#     par(opar)
# }

## ----imageMap, message=FALSE, results='hide'----------------------------------
getNodeBB <- function(ingraph) {
    xypos <- getNodeXY(ingraph)
    hts <- getNodeHeight(ingraph)
    wdsR <- getNodeRW(ingraph)
    wdsL <- getNodeLW(ingraph)
    llx <- xypos$x - wdsL
    lly <- xypos$y - (hts / 2)
    urx <- xypos$x + wdsR
    ury <- xypos$y + (hts / 2)
    cbind(llx, lly, urx, ury)
}
CCbb <- getNodeBB(gCClo)
## now we need to adjust the graphBB to the plot region
bbG <- boundBox(gCClo)
llx <- getX(botLeft(bbG))
lly <- getY(botLeft(bbG))
urx <- getX(upRight(bbG))
ury <- getY(upRight(bbG))
## FIXME: work in progress here - we need to do a bit of mapping
## to get the right user coords on the jpg file...
if (interactive()) {
    jpeg("mygraph.jpg", quality = 100, width = urx - llx, height = ury - lly)
    opar <- par(plt = c(0, 1, 0, 1), xpd = NA)
    plotPieChart(gCClo, ctmat)
    par(opar)
    dev.off()
}
## FIXME (wh 20.1.2005): better to use imageMap function for Ragraph objects
## in the package Rgraphviz instead.
if (require("geneplotter")) {
    con <- openHtmlPage("example", "An Image Map")

    imageMap(CCbb, con,
        tags = list(
            TITLE = getNodeNames(gCClo),
            HREF  = rep("", length(nodes(gCC)))
        ),
        imgname = "mygraph.jpg"
    )

    closeHtmlPage(con)
}

## ----multip-------------------------------------------------------------------
igenes <- unlist(mget(gN[1:10], hgu95av2ENTREZID))
igenes <- igenes[!is.na(igenes)]
igenes <- as.character(igenes)
psets <- mget(igenes, revmap(hgu95av2ENTREZID))
psets <- sapply(psets, function(x) x[1])
GOs <- mget(psets, hgu95av2GO, ifnotfound = NA)
gBP <- sapply(GOs, getOntology, "BP")
## how many terms

sum(sapply(gBP, length))

## drop those with no BP annotations
hasBP <- sapply(gBP, function(x) length(x) > 0 && !is.na(x[1]))
gBP <- gBP[hasBP]
ggs <- lapply(igenes[hasBP], makeGOGraph, "BP", chip = "hgu95av2.db")
## you could also do ggx = lapply(gBP, GOGraph, GOBPPARENTS)
## and should get the same thing

simatM1 <- matrix(1, nr = sum(hasBP), nc = sum(hasBP))
for (i in 1:sum(hasBP)) {
    for (j in 1:sum(hasBP)) {
        if (i == j) {
            next
        }   else {
            simatM1[i, j] <- simUI(ggs[[i]], ggs[[j]])
        }
    }
}
library("RBGL")
simatM2 <- matrix(1, nr = sum(hasBP), nc = sum(hasBP))
for (i in 1:sum(hasBP)) {
    for (j in 1:sum(hasBP)) {
        if (i == j) {
            next
        }   else {
            simatM2[i, j] <- simLP(ggs[[i]], ggs[[j]])
        }
    }
}

## ----leavesEx-----------------------------------------------------------------
gCC.leaves <- leaves(gCC, "in")
length(gCC.leaves)
gCC.leaves[1:5]

 reduce2Degreek <- function(graph, k) {
    if(edgemode(graph) != "undirected")
       stop("only undirected graphs for now")
    done <- FALSE
    while(!done) {
        degs <- degree(graph)
        dgsub <- degs[degs> (k-1)]
        if( length(dgsub) == 0 )
           stop("no nodes left :-(")
        if( length(degs) == length(dgsub))
           done <- TRUE
        else
           graph <- subGraph(names(dgsub), graph)
   }
   return(graph)
  }

 isTriad <- function(x, y, z, elz, ely) {
   if( all(c(x,y) %in% elz ) && all(c(x,z) %in%  ely) )
     return(TRUE)
   return(FALSE)
 }

 enumPairs <- function(iVec) {
   leni <- length(iVec)
   if(leni < 2) return(vector(mode(iVec), length=0))
   eP <- vector("list", length=choose(leni, 2)/2)
   k<-1
   for( i in 1:(leni-1) ) {
     for(j in (i+1):leni) {
       eP[[k]] <- c(iVec[i], iVec[j])
       k <- k+1
     }
   }
   return(eP)
  }


 triadCensus <- function(graph) {
   g1 <- reduce2Degreek(graph, 2) ##all members have to have degree 2
   triads <- NULL
   k <- 1
   el <- edges(g1)
   for( n1 in nodes(g1) ) {
      for(j in enumPairs(el[[n1]])) {
        if( isTriad(n1, j[1], j[2], el[[j[2]]], el[[j[1]]]) ) {
            cand <- sort(c(n1,j))
            dupd <- sapply(triads, function(x) all(x==cand))
            if( length(triads)>1 && any(dupd) )
                next
            triads[[k]] <- cand
            k<- k+1
        }
      }
   }
   return(triads)
 }

##Copyright B. Ding, 2004, all rights reserved

##FIXME: I think some of Jianhua's new environments remove all need
##for these functions
## function to get all the parent nodes for "go" (including "go" itself)
##
##NAME of Beiying's function:
##go.all.parents <- function(go)

## function to get all the children nodes for "go" (including itself)
##go.all.children <- function(go)

## function to find all genes annotated at GO including its children nodes
## env1: mapping from LL to GO
## env2: mapping from GO to LL

##go2allgenes <- function(gos,env1=NULL,env2=NULL)
## calculate the distance between two GO nodes by the cardinality of their GO graph intersection divided by the cardinality of their graph union.
##go.dist <- function(go1,go2)


## calculate distance between GO graphs for geneslist1 and geneslist2
## using all GO nodes the genes can be mapped to
## env3: mapping from GO to ALL LL

geneslist.dist.allnode.ll <- function(geneslist1,geneslist2,ontology="BP",
env1=NULL, env2=NULL, env3=NULL)
  {
    geneslist1.graph <- NULL

    for (j in geneslist1)
      {
        geneslist1.gos <- unlist(mget(j,get(env1)))
        geneslist1.ontology <- geneslist1.gos[!is.na(mget(geneslist1.gos,get(paste("GO",ontology,"ID2TERM",sep=""))))]

    #  print(geneslist.ontology)
        if(length(geneslist1.ontology)==0)
          geneslist1.ontology <- NA

        geneslist1.graph <- union(geneslist1.graph,names(go.all.parents(geneslist1.ontology)))
      }

    geneslist2.graph <- NULL

    for (j in geneslist2)
      {
        geneslist2.gos <- unlist(mget(j,get(env1)))
        geneslist2.ontology <- geneslist2.gos[!is.na(mget(geneslist2.gos,get(paste("GO",ontology,"ID2TERM",sep=""))))]

    #  print(geneslist.ontology)
        if(length(geneslist2.ontology)==0)
          geneslist2.ontology <- NA

        geneslist2.graph <- union(geneslist2.graph,names(go.all.parents(geneslist2.ontology)))
      }

    geneslist1.geneslist2.intersect.graph <- intersect(geneslist1.graph,geneslist2.graph)
    geneslist1.geneslist2.intersect.graph <- geneslist1.geneslist2.intersect.graph[!(geneslist1.geneslist2.intersect.graph %in% c("GO:0008150","GO:0003674","GO:0005575","GO:0003673"))]
    #    print(gene.geneslist.intersect.graph)

    if(length(geneslist1.geneslist2.intersect.graph)>0)
      {

        geneslist1.geneslist2.intersect.graph.lls <- list()

        for (j in geneslist1.geneslist2.intersect.graph)
          {
              geneslist1.geneslist2.intersect.graph.lls[[j]] <-
              intersect(unlist(mget(j, get(env3), ifnotfound=NA)), geneslist2)
          }

        geneslist1.geneslist2.intersect.graph.lls <-
        geneslist1.geneslist2.intersect.graph.lls[sapply(
        geneslist1.geneslist2.intersect.graph.lls, function(x) length(x)>0)]

        score <- sum(sapply(geneslist1.geneslist2.intersect.graph.lls,
        function(x)
        length(x))/sapply(mget(names(geneslist1.geneslist2.intersect.graph.lls),
        get(env3), ifnotfound=NA),function(x)
                          length(x)))
    }else{
        score <- 0
    }
    return(score)
}

## significance test for gene.dist.allnode.ll where geneslist1 is the gene set of interest and geneslist2 is the candidate gene list
geneslist.dist.allnode.ll.sig <- function(geneslist1, geneslist2,
                                          geneslist2.pool=NULL,
                                          ontology="BP", n.samp=100,
                                          env1=NULL, env2=NULL,
                                          env3=NULL)
{
    score <- geneslist.dist.allnode.ll(geneslist1, geneslist2,
                                       ontology=ontology, env1, env2, env3)
                                        # print(score)
    score.null <- NULL

    for ( i in 1:n.samp) {
        print(i)
        geneslist2.null <- sample(geneslist2.pool, length(geneslist2),
                                  FALSE)
                                        # print(geneslist2.null)
        score.null[i] <- geneslist.dist.allnode.ll(geneslist1,
                      geneslist2.null, ontology=ontology, env1, env2, env3)
    }

    pval <- mean(score<=score.null)
    return(list(score=score, pval=pval))
  }

##Copyright R. Gentleman 2004, all rights reserved
##some functions to do the shortest path analysis

shortestPath <- function(g, GOnode, mapfun=NULL, chip=NULL)
{
    if( !is(g, "graph") )
        stop("first argument must be a graph")

    if( edgemode(g) != "undirected" )
        stop("only undirected graphs for now, sorry")

    if( !is.character(GOnode) || length(GOnode) > 1 )
        stop("bad GO node description")

    ##obtain the LLIDs at the GO term
    go2egfun <- .get_eg_to_go_fun(mapfun, chip, reverse=TRUE)
    LLs <- unique(go2egfun(GOnode)[[1]])

    m1 <- match(LLs, nodes(g))
    notthere <- LLs[is.na(m1)]
    there <- LLs[!is.na(m1)]
    if( length(there) <= 1 )
        stop("no nodes correspond to the specified term")

    nthere <- length(there)
    rval <- vector("list", (nthere*(nthere-1))/2)
    nms <- rep("", (nthere*(nthere-1))/2)
    k <- 1
    there <- as.character(there)
    for(i in 1:(nthere-1))
        for(j in (i+1):nthere) {
            nms[k] <- paste(there[i], there[j], sep=":")
            rval[[k]] <- sp.between(g,there[i], there[j])
            k <- k+1
        }
    names(rval) <- nms
    return(list(shortestpaths=rval, nodesUsed=there,
                nodesNotUsed=notthere))
}

compGdist <- function(g, whNodes, verbose=FALSE) {
    nNodes = length(whNodes)
    if(nNodes <= 1) 
        stop("can only compute distances between two or more nodes")
    if( any( !(whNodes %in% nodes(g))) )
        stop("specified nodes are not in the graph")
    tfmatrix <- matrix(0, nrow= nNodes, ncol=nNodes)
    for(i in 1:nNodes) {
        vvX <- dijkstra.sp(g, whNodes[i])
        tfmatrix[i,] <- vvX$distance[whNodes]
        if( verbose)
            print(paste("Processing node", i, "of", nNodes))
    }
    dimnames(tfmatrix) <- list(whNodes, whNodes)
    tfmatrix
}

compCorrGraph <- function(eSet, k=1, tau=0.6) {
    if( tau < 0 || tau > 1 ) stop("bad tau value")
    cB <- abs(cor(t(exprs(eSet))))
    cB[cB < tau] <- 0
    whE = cB>0
    ##FIXME: any two genes whose exprssion values are perfectly
    ##correlated will get dropped - seems pretty unlikely
    cB[whE] <- ((1-cB)[whE])^k
    ##set diag to zero, since we do not want self-edges
    diag(cB) = 0
    requireNamespace("SparseM", quietly=TRUE) || stop("need SparseM package")
    v1<-SparseM::as.matrix.csr(cB, nrow=dim(cB)[1], ncol=dim(cB)[2])
    sparseM2Graph(v1, featureNames(eSet))
}

 ##given a matrix and a logical vector return two lists
 ##the row names and the column names for each selected entry

 idx2dimnames = function(x, idx) {
     if(length(idx) != length(x) )
         stop("x and idx different lengths")
     rowV = row(x)[idx]
     colV = col(x)[idx]
     return(list(rowNames = dimnames(x)[[1]][rowV],
                 colNames = dimnames(x)[[2]][colV]))
 }

##take as input a distance matrix
##if the number of infs is 1 less than the number of rows
##then that nod is not connected
 notConn = function(dists) {
   allInf = (dists==Inf)
   dns = idx2dimnames(dists, allInf)
   tab1 = table(dns[[1]])
   whAll = tab1[tab1 == (nrow(dists)-1)]
   if( any(tab1 != (nrow(dists)-1) & tab1 != length(whAll)) )
      warning("inf dists seem to be wrong")
   return(names(whAll))
}

library("hgu95av2.db")

makeSimpleGOHyperGParams <- function() {
    set.seed(344)
    probeIds <- ls(hgu95av2ENTREZID)
    randProbeIds <- sample(probeIds, 500)
    ## This is "wrong", should unlist, but the code
    ## should catch/correct it.  The right way is to
    ## unlist the mget result.
    entrezUniverse <- mget(randProbeIds, hgu95av2ENTREZID,
                           ifnotfound=NA)
    entrezUniverse <- entrezUniverse[!is.na(entrezUniverse)]
    selectedEntrezIds <- sample(entrezUniverse, 30)
    params <- new("GOHyperGParams",
                  geneIds=selectedEntrezIds, 
                  universeGeneIds=entrezUniverse,
                  annotation="hgu95av2", 
                  ontology="BP",
                  pvalueCutoff=0.05,
                  conditional=TRUE,
                  testDirection="over")
    params
}
    


test_hyperGTest_regression2 <- function() {
    p <- makeSimpleGOHyperGParams()
    p@conditional <- FALSE
    res <- hyperGTest(p)

    ## Verify result is same using geneCategoryHyperGeoTest
    ## from Category
    res2 <- Category::hyperGTest(p)
    checkEquals(pvalues(res), pvalues(res2))
    checkEquals(geneCounts(res), geneCounts(res2))
    checkEquals(universeCounts(res), universeCounts(res2))
    checkEquals(universeMappedCount(res), universeMappedCount(res2))
    checkEquals(geneMappedCount(res), geneMappedCount(res2))
    checkEquals(annotation(res), annotation(res2))
    checkEquals(testName(res), testName(res2))

    ## Regression tests
    checkEquals("hgu95av2", annotation(res))
    checkEquals(c("GO", "BP"), testName(res))
}

---
title: "Visualizing and Distances Using GO"
authors:
  - name: "R. Gentleman"
  - name: "Sonali Kumari"
    affiliation: "Vignette translation from Sweave to Rmarkdown / HTML"
date: "`r format(Sys.time(), '%B %d , %Y')`"
vignette: >
  %\VignetteIndexEntry{Visualizing and Distances Using GO}
  %\VignetteDepends{Biobase,genefilter,graph,GO.db,hgu95av2.db,xtable,multtest}
  %\VignetteSuggests{Rgraphviz}
  %\VignetteKeywords{EDA, graphs, ontology, visualization}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignettePackage{GOstats}
output:
  BiocStyle::html_document
bibliography: GOstats.bib
link-citations: yes
csl: bioinformatics.csl
---

# Introduction {#introduction .unnumbered}

The basic characteristics of the GO [@GO] data are described in @GO and the
interested reader is referred there for more details. In this paper we assume
that readers are familiar with the basic DAG structure of GO and with the
mappings of genes to GO terms that are provide by GOA [@GOA]. We examine  some
of the different visualization problems and data analytic problems that can be
addressed using these data. To make some of the concepts concrete and to provide
extensive, but related examples we will make use of a microarray data set
[@Chiaretti2004].

We begin with brief descriptions and reminders of some of the important
concepts, as well as of our example data. Once that has been done we will then
consider several different bioinformatic tasks that are of interest and show how
these tasks can be carried out using software from the Bioconductor Project.

```{r Setup, echo=FALSE, results='hide', message=FALSE}
library("Biobase")
library("annotate")
library("GOstats")
library("xtable")
library("multtest")
library("Rgraphviz")
library("hgu95av2.db")
library("GO.db")
library("genefilter")
library("ALL")
```

The results reported in this document are based on `r pkg_ver("hgu95av2.db")`
version of the `r Biocpkg("hgu95av2.db")` data package and on `r pkg_ver("GO.db")`
of the `r Biocpkg("GO.db")` package. If you have different versions your results
will probably be different.

## Graphs

The relationships between different GO terms, within a specific ontology are
represented in the form of a directed acyclic graph. The leaves of this graph
represent the most specific terms and their are edges from a specific term
(child) to all less specific terms (each is a parent). The *induced* GO graph
is the graph that obtains from taking a set of GO terms and finding all parents
of those terms, and so on until the root node has been obtained. Given any set
of genes one may use the mappings provided by GOA [@GOA] to find the most
specific set of GO terms associated with those genes and hence, from those,
to the induced GO graph. Thus we can speak of the induced GO graph either from
the perspective of selected genes or from the perspective of selected terms.

We note that there is a form of algebra possible on these induced GO graphs. If
$G_1$ and $G_2$ are two different GO graphs from the same ontology then we can
form their union, $G_1 \cup G_2$ which is the union of the nodes in the two
graphs together with the union of the edge sets. We note that this is a bit
different than the usual mathematical definition of the union of two graphs.
Other set operations such as intersection and complement are also easy to define
and implement and they will play a role in our subsequent discussions.

## Some problems

One should be aware that our state of biological knowledge is constantly
changing and that the terms, their relationships and the genes mapped to them
are constantly changing and being updated. The meta-data used to prepare this
report will likely be out of date in less than one year. Hence, one must try to
ensure that the mappings being used are up to date (Bioconductor meta-data
packages are built quarterly).

Another practical issue that needs some attention is consideration of the
incompleteness of the data. Not all genes (or all transcripts) are assayed and
not all genes are well enough understood to accurately annotate them at the
existing GO nodes; to say nothing of the issues regarding the evolving nature of
GO discussed above. For each problem the data analyst will need to consider
what, if any, biases these issues might raise.

In all comparisons and gene list developments reported in the remainder of this
paper we restrict attention to those genes which satisfy two conditions first
they were included on the chip being used and second they have a GO annotation
for the ontology of interest.

## An Example {#sec:example}

To demonstrate some of the tools that are included in the `r Biocpkg("GOstats")`
package we consider expression data from 79 samples from patients with acute
lymphoblastic leukemia (ALL) that were investigated using HGU95AV2 Affymetrix
GeneChip arrays [@Chiaretti2004]. The data were normalized using quantile
normalization and expression estimates were computed using RMA [@Irizarry2003].
We will consider the subset of these patients that have B-cell ALL and within
those we will further restrict our interest to comparing three groups. Those
with BCR/ABL (this is a translocation between chromosomes 9 and 22), those with
ALL1/AF4 (this is a translocation between chromosomes 4 and 11) and those with
no detected cytogenetic abnormalities (labeled NEG). There are 37 patients with
BCR/ABL, 10 with ALL1/AF4 and 42 from the NEG group.

We will examine genes that are differentially expressed between these different
phenotypes and then make use of GO and other meta-data resources to help
understand the genes that were selected. The actual method of determining
differentially expressed genes is in some sense irrelevant to the subsequent
analysis using GO and readers can easily substitute their own favorite methods.
We refer readers to @WileyEncy for a general discussion of some of the issues
involved in gene filtering.

```{r Example, results='hide', echo=FALSE}
## subset of interest: 37+42 samples
data(ALL)
eset <- ALL[, intersect(
    grep("^B", as.character(ALL$BT)),
    which(as.character(ALL$mol) %in% c("BCR/ABL", "NEG", "ALL1/AF4"))
)]
MB <- factor(eset$mol.bio)

## intensities above 100 in at least 7 of the samples
f1 <- kOverA(7, log2(100))
f2 <- function(x) {
    gps <- split(2^x, MB)
    mns <- sapply(gps, mean)
    if (max(mns) - min(mns) < 100) FALSE else TRUE
}
ff <- filterfun(f1, f2)
selected <- genefilter(eset, ff)
sum(selected)
esetSub <- eset[selected, ]
```

Our approach is fairly simplistic. First, we require that genes be expressed in
a reasonable set of the samples. Since our smallest group, those with ALL1/AF4,
has only 10 samples we will require that a gene be expressed in at least 7
samples to be deemed interesting. For our purposes we consider a gene to be
expressed if its value is greater than 100. We also introduce a second selection
criteria that is based on differences between the group means. We require that
the difference between the smallest group mean and the largest be larger than
100 units. Applying both of these transformations yielded `r sum(selected)`
probes that were differentially expressed.

```{r mtFstats, results='hide', echo=FALSE}
pvCutOff <- 0.05

Fstats <- mt.maxT(exprs(esetSub), as.numeric(MB) - 1,
                  B = 1000, test = "f"
)
eSet <- esetSub[Fstats$index[Fstats$adjp < pvCutOff], ]
gN <- featureNames(eSet)
lls <- unlist(mget(gN, hgu95av2ENTREZID, ifnotfound = NA))
eS <- eSet[!duplicated(lls), ]
gN <- featureNames(eS)
lls <- unlist(mget(gN, hgu95av2ENTREZID, ifnotfound = NA))
lls <- as.character(lls[!is.na(lls)])
syms <- unlist(mget(gN, hgu95av2SYMBOL, ifnotfound = NA))
```

Next restricting attention to just these `r sum(selected)` probes we
use the `r Rpackage("mt.maxT")` function from the `r Biocpkg("multtest")`
package to compute adjusted $p$-values for F-test comparisons.  We found
`r sum(Fstats$adjp < pvCutOff)` of these to have adjusted $p$-values less
than `r round(pvCutOff, 2)`. We will deem these to be the differentially
expressed genes and will now turn our attention to using GO to gain a
better understanding of the associated genes.  Before going further 
though, we note that these `r sum(Fstats$adjp < pvCutOff)` probes map to
`r sum(!duplicated(lls))` distinct Entrez Gene identifiers and since
our GO mappings are based on Entrez Gene we further reduce our attention
to these. We kept the first (in index order) of each of the duplicated
probes for any analyses or computations that require gene expression
data and note that others may want to use different conventions.

# Visualization

Using the genes selected in Section \@ref(sec:example) we can construct the
induced GO graph for any one of the three different Ontologies. In Figures
[1](#fig:mf), [2](#fig:bp) and [3](#fig:cc) we present these three different
graphs. Of course one would like to add some of the experimental data and
information to these plots.

```{r inducedGO, echo=FALSE, results='hide'}
gMF <- makeGOGraph(lls, "MF", chip = "hgu95av2")
gBP <- makeGOGraph(lls, "BP", chip = "hgu95av2")
gCC <- makeGOGraph(lls, "CC", chip = "hgu95av2")
nMF <- list()
lbs <- rep("", length(nodes(gMF)))
names(lbs) <- nodes(gMF)
nMF$label <- lbs
nBP <- list()
lbs <- rep("", length(nodes(gBP)))
names(lbs) <- nodes(gBP)
nBP$label <- lbs
nCC <- list()
lbs <- rep("", length(nodes(gCC)))
names(lbs) <- nodes(gCC)
nCC$label <- lbs
if (require("Rgraphviz")) {
    gMFlo <- agopen(gMF, "gMF")
    gBPlo <- agopen(gBP, "gBP")
    gCClo <- agopen(gCC, "gCC")
}
```

The nodes in these different graphs can be colored according to different
criteria. In @Storch they considered using color for the nodes. The studied
differences in gene expression between heart and liver in mice. Their Figure 2
presents data that were assembled in roughly the following manner. Genes of
interest were mapped to terms in the Biological Process arm of GO @GO. They then
colored the nodes in this tree red if the associated genes were expressed in
heart, green if they were expressed in liver and yellow if they were expressed
in both. The figure itself is quite helpful and clearly displays the data
informatively.

This is an interesting idea and we amplify it a bit here. In our example we have
three groups and we might want to consider coloring the nodes differently. For
each node in the induced graph there is one or more genes from our list of
interesting genes annotated at that node (due to the way we constructed the
graph). We could color the nodes according to the phenotype (ALL1/AF4, BCR/ABL,
or NEG) that had the highest mean (if there is more than one gene then we might
use the multiplicity of highest means). Or, possibly of more interest would be
to categorize the genes (again we could use which phenotype had the highest
mean) and render the nodes using pie charts.

<a id="fig:mf"></a>
```{r GOMF, fig=TRUE, echo=FALSE, warning=FALSE, fig.cap = " The induced MF GO graph for the selected genes."}
if (require("Rgraphviz")) {
    plot(gMFlo, nodeAttrs = nMF, main = "Molecular Function")
}
```

<a id="fig:bp"></a>
```{r GOBP, fig=TRUE, echo=FALSE, warning=FALSE, fig.cap = " The induced BP GO graph for the selected genes."}
if (require("Rgraphviz")) {
    plot(gBPlo, nodeAttrs = nBP, main = "Biological Process")
}
```

<a id="fig:cc"></a>
```{r GOCC, fig=TRUE, echo=FALSE, warning=FALSE, fig.cap = "The induced CC GO graph for the selected genes."}
if (require("Rgraphviz")) {
    plot(gCClo, nodeAttrs = nCC, main = "Cellular Component")
}
```

The computations are quite straightforward. First we find the maximum mean for
each gene. We see that BCR/ABL has the most maxima, followed by the ALL1/AF4
samples and finally the NEGs.

```{r findhigestmeans}
mns <- apply(exprs(eS), 1, function(x) sapply(split(x, MB), mean))
whismax <- apply(mns, 2, function(x) match(max(x), x))
maxNames <- names(mns[, 1])[whismax]
names(maxNames) <- names(whismax)
table(maxNames)
```

We next need to associate each GO term with the set of these genes and
classify them according to which phenotype has the highest mean. Since
the CC ontology has the fewest nodes in the induced graph we will use it
for the examples here -- the reader may want to adapt the code to deal
with the other two ontologies.

```{r genesToGO}
CCnodes <- nodes(gCC)
nodes2affy <- mget(CCnodes, hgu95av2GO2ALLPROBES, ifnotfound = NA)
cts <- sapply(nodes2affy, function(x) {
    wh <- names(whismax) %in% x
    table(maxNames[wh])
})
all1cts <- sapply(cts, function(x) x["ALL1/AF4"])
all1cts <- ifelse(is.na(all1cts), 0, all1cts)
## put nice names on
names(all1cts) <- names(cts)
bcrcts <- sapply(cts, function(x) x["BCR/ABL"])
bcrcts <- ifelse(is.na(bcrcts), 0, bcrcts)
names(bcrcts) <- names(cts)
negcts <- sapply(cts, function(x) x["NEG"])
negcts <- ifelse(is.na(negcts), 0, negcts)
names(negcts) <- names(cts)
ctmat <- cbind(all1cts, bcrcts, negcts)
```

Now that we have assembled the requisite counts we are now ready to layout and
render the graph. The code in the following code chunk does this. We first do a
layout to get the node positions and then write a function that will draw each
node as a piechart and color the components the requested colors.

```{r layoutandrender, eval=FALSE, warning=FALSE, results="hide"}
if (require(Rgraphviz)) {
    opar <- par(xpd = NA)
    plotPieChart <- function(curPlot, counts, main) {
        renderNode <- function(x) {
            force(x)
            y <- x * 100 + 1
            function(node, ur, attrs = list(), radConv = 1) {
                nodeCenter <- getNodeCenter(node)
                pieGlyph(y,
                    xpos = getX(nodeCenter),
                    ypos = getY(nodeCenter),
                    radius = getNodeRW(node),
                    col = c("blue", "green", "red")
                )
            }
        }
        drawing <- vector(mode = "list", length = nrow(counts))
        for (i in 1:length(drawing)) {
            drawing[[i]] <- renderNode(counts[i, ])
        }
        if (missing(main)) {
            main <- "Example Pie Chart Plot"
        }

        plot(curPlot, drawNode = drawing, main = main)

        legend(
            x = "bottomleft", legend = c("ALL1/AF4", "BCR/ABL", "NEG"),
            fill = c("blue", "green", "red")
        )
    }
    plotPieChart(gCClo, ctmat)
    par(opar)
}
```

We can see a number of interesting features in this plot. For example, long
sequences of single colored nodes - as set of blue nodes on the right and
another set of red nodes on the left. We might want to investigate these and see
what cellular components they are related to.

We can make use of the `imageMap` function to provide tool tips for our graph.

```{r imageMap, message=FALSE, results='hide'}
getNodeBB <- function(ingraph) {
    xypos <- getNodeXY(ingraph)
    hts <- getNodeHeight(ingraph)
    wdsR <- getNodeRW(ingraph)
    wdsL <- getNodeLW(ingraph)
    llx <- xypos$x - wdsL
    lly <- xypos$y - (hts / 2)
    urx <- xypos$x + wdsR
    ury <- xypos$y + (hts / 2)
    cbind(llx, lly, urx, ury)
}
CCbb <- getNodeBB(gCClo)
## now we need to adjust the graphBB to the plot region
bbG <- boundBox(gCClo)
llx <- getX(botLeft(bbG))
lly <- getY(botLeft(bbG))
urx <- getX(upRight(bbG))
ury <- getY(upRight(bbG))
## FIXME: work in progress here - we need to do a bit of mapping
## to get the right user coords on the jpg file...
if (interactive()) {
    jpeg("mygraph.jpg", quality = 100, width = urx - llx, height = ury - lly)
    opar <- par(plt = c(0, 1, 0, 1), xpd = NA)
    plotPieChart(gCClo, ctmat)
    par(opar)
    dev.off()
}
## FIXME (wh 20.1.2005): better to use imageMap function for Ragraph objects
## in the package Rgraphviz instead.
if (require("geneplotter")) {
    con <- openHtmlPage("example", "An Image Map")

    imageMap(CCbb, con,
        tags = list(
            TITLE = getNodeNames(gCClo),
            HREF  = rep("", length(nodes(gCC)))
        ),
        imgname = "mygraph.jpg"
    )

    closeHtmlPage(con)
}
```

# GO induced Distances

Another idea that has seen some exploration is the notion of defining distances
between two genes in terms of the similarity of their GO annotations. We explore
some of the suggestions and provide some guidance for those who would like to
explore these ideas.

It seems reasonable to treat the three ontologies separately. Even within an
ontology a single gene might be annotated at several GO terms. Some mechanisms
for dealing with this issue would be helpful. For each gene we could define the
graph to be that which is induced by all GO terms to which the gene is mapped,
within the ontology of interest. Then use the between graph distances as already
described. One could define distance as some combination of the annotation
specific distances. For example, if gene 1 is annotated at 2 terms and gene 2 is
annotated at three terms there are 6 distances to be computed and some
functional of these 6 distances will be reported.

There are two basic strategies that have been widely used to define distances,
or similarities, between genes based on GO annotation. One, is to make use of
the graph structure that is associated with each pair of genes. If the two
graphs are very similar then the two genes are presumed to also be quite
similar. The second general strategy is to use the information content in GO as
a basis for assigning similarity. Readers should consult @Lordetal for a general
review of the area and for some suggestions. Code contributions that implement
some of their methods would be most welcome.

Two methods based on graph similarity have been implemented in `r Biocpkg("GOstats")`
they are `simUI` and `simLP`. The first uses the number of nodes that the two 
graphs have in common divided by the number of nodes in the union of the two node
sets. Hence these similarities are bounded between 0 and 1 with genes that are more
similar having values near one.

For `simLP` the similarity measure is the depth of the longest shared path from
the root node. Two genes that are both quite specific and similar should have
long shared paths, while those that have less in common should have relatively
short shared paths.

Of course neither is perfect and there are many cases where they seem to fail to
uncover similarity and others where they suggest a similarity that does not
exist. Further investigation and comparison with other methods is certainly
warranted.

To compare different genes with respect to their GO annotation you may use
`simLL` which first computes the induced GO graph for each of its two, user
supplied, Entrez Gene IDs and then calls either `simLP` or `simUI`. Users can
restrict the mappings from Entrez Gene IDs to GO IDs based on evidence codes.

Some further work is needed to correctly address two outstanding issues. First,
in some cases an Entrez Gene identifier is mapped to several GO terms and in the
current implementation these are merely combined into one large graph. However,
there may be some benefits in considering the comparison of each induced graph
separately. For example, suppose that we have a gene, $G_1$ that performs a
specific molecular function and we want to know which other genes are reasonably
similar to it. We might not care if those other genes also do something else and
so in this case we would probably use each mapping separately.

The second issue that needs to be addressed at some time is to include
functionality that would allow the user to construct a GO graph based only on
*is-a* relationships or only on *partof* relationships.

In the next code chunk we take the first 10 genes selected and construct a set
of similarity measures between all pairs of genes with respect to the BP
ontology.

```{r multip}
igenes <- unlist(mget(gN[1:10], hgu95av2ENTREZID))
igenes <- igenes[!is.na(igenes)]
igenes <- as.character(igenes)
psets <- mget(igenes, revmap(hgu95av2ENTREZID))
psets <- sapply(psets, function(x) x[1])
GOs <- mget(psets, hgu95av2GO, ifnotfound = NA)
gBP <- sapply(GOs, getOntology, "BP")
## how many terms

sum(sapply(gBP, length))

## drop those with no BP annotations
hasBP <- sapply(gBP, function(x) length(x) > 0 && !is.na(x[1]))
gBP <- gBP[hasBP]
ggs <- lapply(igenes[hasBP], makeGOGraph, "BP", chip = "hgu95av2.db")
## you could also do ggx = lapply(gBP, GOGraph, GOBPPARENTS)
## and should get the same thing

simatM1 <- matrix(1, nr = sum(hasBP), nc = sum(hasBP))
for (i in 1:sum(hasBP)) {
    for (j in 1:sum(hasBP)) {
        if (i == j) {
            next
        }   else {
            simatM1[i, j] <- simUI(ggs[[i]], ggs[[j]])
        }
    }
}
library("RBGL")
simatM2 <- matrix(1, nr = sum(hasBP), nc = sum(hasBP))
for (i in 1:sum(hasBP)) {
    for (j in 1:sum(hasBP)) {
        if (i == j) {
            next
        }   else {
            simatM2[i, j] <- simLP(ggs[[i]], ggs[[j]])
        }
    }
}
```

You can plot these two measures against each other and compare them. We hope
that the two measures are somewhat positively correlated.

# Operations on GO Graphs

You should refer to the documentation for the `r Biocpkg("graph")` and 
`r Biocpkg("RBGL")` packages for details on how to manipulate *graph* 
instances. Here we show how to identify the leaves of a GO graph.

```{r leavesEx}
gCC.leaves <- leaves(gCC, "in")
length(gCC.leaves)
gCC.leaves[1:5]
```

# References

---
title: "How To Use GOstats and Category to do Hypergeometric testing
with unsupported model organisms"
authors:
  - name: "M. Carlson"
  - name: "Sonali Kumari"
    affiliation: "Vignette translation from Sweave to Rmarkdown / HTML"
date: "`r format(Sys.time(), '%B %d , %Y')`"
package: GOstats
vignette: >
  %\VignetteIndexEntry{How To Use GOstats and Category to do Hypergeometric testing with unsupported model organisms}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document
---

# Introduction

This vignette is meant as an extension of what already exists in the
[GOstatsHyperG](GOstatsHyperG.html) vignette. It is intended to explain how a
user can run hypergeometric testing on GO terms or KEGG terms when the organism
in question is not supported by an annotation package. The 1st thing for a user
to determine then is whether or not their organism is supported by an organism
package through `r Biocpkg("AnnotationForge")`. In order to do that, they need
only to call the available.dbschemas() method from `r Biocpkg("AnnotationForge")`.

```{r available Schemas, message=FALSE, warning=FALSE}
library("AnnotationForge")
available.dbschemas()
```

If the organism that you are using is listed here, then your organism is
supported. If not, then you will need to find a source or GO (org KEGG) to gene
mappings. One source for GO to gene mappings is the blast2GO project. But you
might also find such mappings at Ensembl or NCBI. If your organism is not a
typical model organism, then the GO terms you will find are probably going to be
predictions based on sequence similarity measures instead of direct
measurements. This is something that you might want to bear in mind when you
draw conclusions later.

In preparation for our subsequent demonstrations, lets get some data to work
with by borrowing from an organism package. We will assume that you will use
something like `read.table` to load your own annotation packages into a
data.frame object. The starting object needs to be a data.frame with the GO Id's
in the 1st col, the evidence codes in the 2nd column and the gene Id's in the
3rd.

```{r Acquire annotation data, message=FALSE}
library("org.Hs.eg.db")
frame = toTable(org.Hs.egGO)
goframeData = data.frame(frame$go_id, frame$Evidence, frame$gene_id)
head(goframeData)
```

## Preparing GO to gene mappings

When using GO mappings, it is important to consider the data structure of the GO
ontologies. The terms are organized into a directed acyclic graph. The structure
of the graph creates implications about the mappings of less specific terms to
genes that are mapped to more specific terms. The `r Biocpkg("Category")` and 
`r Biocpkg("GOstats")` packages normally deal with this kind of complexity by 
using a special GO2All mapping in the annotation packages. You won't have one of
those, so instead you will have to make one. `r Biocpkg("AnnotationDbi")`
provides some simple tools to represent your GO term to gene mappings so that
this process will be easy. First you need to put your data into a GOFrame
object. Then the simple act of casting this object to a GOAllFrame object will
tap into the GO.db package and populate this object with the implicated GO2All
mappings for you.

```{r transformGOFrame, message=FALSE, warning=FALSE}
goFrame=GOFrame(goframeData,organism="Homo sapiens")
goAllFrame=GOAllFrame(goFrame)
```

In an effort to standardize the way that we pass this kind of custom
information around, we have chosen to support geneSetCollection objects
from `r Biocpkg("GSEABase")` package. You can generate one of these objects
in the following way:

```{r Make GSC, message=FALSE}
library("GSEABase")
gsc <- GeneSetCollection(goAllFrame, setType = GOCollection())
```

## Setting up the parameter object

Now we can make a parameter object for `r Biocpkg("GOstats")` by using a special
constructor function. For the sake of demonstration, I will just use all the EGs
as the universe and grab some arbitrarily to be the geneIds tested. For your
case, you need to make sure that the gene IDs you use are unique and that they
are the same type for both the universe, the geneIds and the IDs that are part
of your geneSetCollection.

```{r make parameter, message=FALSE}
library("GOstats")
universe = Lkeys(org.Hs.egGO)
genes = universe[1:500]
params <- GSEAGOHyperGParams(name="My Custom GSEA based annot Params", 
                             geneSetCollection=gsc, 
                             geneIds = genes, 
                             universeGeneIds = universe, 
                             ontology = "MF", 
                             pvalueCutoff = 0.05, 
                             conditional = FALSE, 
                             testDirection = "over")
```

And finally we can call `hyperGTest` in the same way we always have before.

```{r call HyperGTest}
Over <- hyperGTest(params)
head(summary(Over))
```

## Preparing KEGG to gene mappings

This is much the same as what you did with the GO mappings except for two
important simplifications. First of all you will no longer need to track
evidence codes, so the object you start with only needs to hold KEGG Ids and
gene IDs. Seconly, since KEGG is not a directed graph, there is no need for a
KEGG to All mapping. Once again I will borrow some data to use as an example.
Notice that we have to put the KEGG Ids in the left hand column of our initial
two column data.frame.

```{r KEGGFrame object}
frame = toTable(org.Hs.egPATH)
keggframeData = data.frame(frame$path_id, frame$gene_id)
head(keggframeData)
keggFrame=KEGGFrame(keggframeData,organism="Homo sapiens")
```

The rest of the process should be very similar.

```{r KEGG Parameters}
gsc <- GeneSetCollection(keggFrame, setType = KEGGCollection())
universe = Lkeys(org.Hs.egGO)
genes = universe[1:500]
kparams <- GSEAKEGGHyperGParams(name="My Custom GSEA based annot Params", 
                               geneSetCollection=gsc, 
                               geneIds = genes, 
                               universeGeneIds = universe,  
                               pvalueCutoff = 0.05, 
                               testDirection = "over")
kOver <- hyperGTest(params)
head(summary(kOver))
```

```{r info}
sessionInfo()
```

---
title: "How To Use GOstats Testing Gene Lists for GO Term Association"
author:
  - name: S. Falcon 
  - name: R. Gentleman
  - name: Dennis Ndubi
    affiliation: "Vignette translation from Sweave to R Markdown / HTML"
date: "`r format(Sys.time(), '%B %d, %Y')`"
package: GOstats
bibliography: GOstats.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{Hypergeometric Tests Using GOstats}
  %\VignetteDepends{ALL, GO.db, annotate, hgu95av2.db, genefilter, RColorBrewer, xtable, Rgraphviz}
  %\VignetteKeywords{Hypergeometric, GO, ontology}
  %\VignettePackage{GOstats}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    number_sections: yes
    toc: no
    toc_depth: 4
csl: bioinformatics.csl
---

# Introduction

The `r Biocpkg("GOstats")` package has extensive facilities for testing
the association of Gene Ontology (GO) @GO terms to genes in a gene
list. You can test for both over and under representation of GO terms
using either the standard Hypergeometric test or a conditional
Hypergeometric test that uses the relationships among the GO terms for
conditioning (similar to that presented in Alexa et al. [-@Alexa06]).

In this vignette we describe the preprocessing required to construct
inputs for the main testing function, `hyperGTest`, the algorithms used,
and the structure of the return value. We use a microarray data set
[@Chiaretti2004] from a clinical trial in acute lymphoblastic leukemia
(ALL) to work an example analysis. In the ALL data, we focus on the
patients with B-cell derived ALL, and in particular on comparing the
group with ALL1/AF4 to those with no observed cytogenetic abnormalities.

To get started, load the packages needed for this analysis:

```{r Setup, echo=TRUE, message=FALSE}
library("ALL")
library("hgu95av2.db")
library("GO.db") 
library("annotate")
library("genefilter") 
library("GOstats") 
library("RColorBrewer")
library("xtable") 
library("Rgraphviz")
```

# Preprocessing and Inputs

To perform an analysis using the Hypergeometric-based tests, one needs
to define a *gene universe* (usually conceptualized as the number of
balls in an urn) and a list of selected genes from the universe. While
it is clear that the selected gene list determines to a large degree the
results of the analysis, the fact that the universe has a large effect
on the conclusions is, perhaps, less obvious.

For microarray data, one can use the unique gene identifiers assayed in
the experiment as the gene universe. However, the presence of a gene on
the array does not necessarily mean much. Some arrays, such as those
from Affymetrix, attempt to include probes for as much of the genome as
possible. Since not all genes will be expressed under all conditions (a
widely held belief is that about 40% of the genome is expressed in any
tissue), it may be sensible to reduce the universe to those that are
expressed.

To identify the set of expressed genes from a microarray experiment, we
propose that a non-specific filter be applied and that the genes that
pass the filter be used to form the universe for any subsequent
functional analyses. Below, we outline the non-specific filtering
procedure used for the example analysis.

Once a gene universe has been established, one can apply any number of
methods to select genes. For the example analysis we use a simple
$t$-test to identify differentially expressed genes among the two
subgroups in the sample population.

```{r universeSizeVsPvalue, eval=TRUE, echo=FALSE, results='hide'}
hg_tester <-function(size){
  numFound <- 10 
  numDrawn <- 400 
  numAtCat <- 40
  numNotAtCat <- size - numAtCat 
  phyper(numFound-1, numAtCat,numNotAtCat, numDrawn, lower.tail=FALSE)
}
pv1000 <- hg_tester(1000)
pv5000 <- hg_tester(5000)
```

It is worth noting that the effect of increasing the universe size with
genes that are irrelevant to the questions at hand, in general, has the
effect of making the resultant $p$-values look more significant. For
example, in a universe of 1000 genes where 400 have been selected,
suppose that a GO term has 40 gene annotations from the universe of
1000. If 10 of the genes in the selected gene list are among the 40
genes annotated at this category, then the Hypergeometric $p$-value is `r round(pv1000,2)`.
However, if the gene universe contained 5000 genes, the $p$-value would
drop to `r round(pv5000,3)`.

## Non-specific filtering

First we load the `ALL` data object and extract the subset of the data
we wish to analyze: subjects with either no cytogenetic abnormality
("NEG") or those with "ALL1/AF4".

```{r bcrAblOrNegSubset, results='hide', echo=TRUE, cache=TRUE}
data(ALL, package="ALL") 
## For this data we can have ALL1/AF4 or BCR/ABL 
subsetType <- "ALL1/AF4"
## Subset of interest: 37BRC/ABL + 42NEG = 79 samples 
Bcell <- grep("^B", as.character(ALL$BT))
bcrAblOrNegIdx <- which(as.character(ALL$mol) %in% c("NEG", subsetType))
bcrAblOrNeg <- ALL[, intersect(Bcell, bcrAblOrNegIdx)]
bcrAblOrNeg$mol.biol = factor(bcrAblOrNeg$mol.biol)
```

We begin our non-specific filtering by removing probe sets that have no
Entrez Gene identifier in our annotation data.

```{r nsFiltering-noEntrez, results='hide', echo=TRUE, cache=TRUE}
##Remove genes that have no entrezGene id 
entrezIds <- mget(featureNames(bcrAblOrNeg), envir=hgu95av2ENTREZID) 
haveEntrezId <- names(entrezIds)[sapply(entrezIds, function(x) !is.na(x))]
numNoEntrezId <- length(featureNames(bcrAblOrNeg)) - length(haveEntrezId) 
bcrAblOrNeg <- bcrAblOrNeg[haveEntrezId, ]
```

Similarly, we remove probe sets for which we have no GO annotation.

```{r nsFiltering-noGO, results='hide', echo=TRUE, cache=FALSE} 
## Remove genes with no GO mapping 
haveGo <- sapply(mget(featureNames(bcrAblOrNeg), hgu95av2GO), 
                 function(x){
                   if (length(x) == 1 && is.na(x)) 
                     FALSE 
                   else TRUE 
                   })
numNoGO <- sum(!haveGo)
bcrAblOrNeg <- bcrAblOrNeg[haveGo, ]
```

Now use the IQR of each probe set across samples to remove those probe
sets that have little variation across samples. Also, since there is an
imbalance of men and women by group, we remove probe sets that measure
genes on the Y chromosome.

```{r nsFiltering-IQR, results='hide', echo=TRUE, cache=TRUE}
## Non-specific filtering based on IQR 
iqrCutoff <- 0.5 
bcrAblOrNegIqr <- apply(exprs(bcrAblOrNeg), 1, IQR) 
selected <- bcrAblOrNegIqr > iqrCutoff

## Drop those that are on the Y chromosome 
## because there is an imbalance of men and women by group 
chrN <- mget(featureNames(bcrAblOrNeg), envir=hgu95av2CHR) 
onY <- sapply(chrN, function(x) any(x=="Y"))
onY[is.na(onY)] <- FALSE
selected <- selected & !onY

nsFiltered <- bcrAblOrNeg[selected, ]
```

Here we ensure that each probe set maps to exactly one Entrez Gene ID.
If multiple probes are found to map to the same Entrez Gene ID, we
select the probe with largest IQR (from the computation above).

```{r nsFiltering-unique, results='hide', echo=TRUE, cache=TRUE}
numNsWithDups <- length(featureNames(nsFiltered))
nsFilteredIqr <- bcrAblOrNegIqr[selected] 
uniqGenes <- findLargest(featureNames(nsFiltered), nsFilteredIqr, 
                         "hgu95av2")
nsFiltered <- nsFiltered[uniqGenes, ] 
numSelected <- length(featureNames(nsFiltered))

##set up some colors 
BCRcols = ifelse(nsFiltered$mol == subsetType, "goldenrod", "skyblue")
cols = brewer.pal(10, "RdBu")
```

Finally, we can define the gene universe we will use for the
Hypergeometric tests.

```{r defineGeneUniverse, echo=TRUE, results='hide', cache=TRUE}
## Define gene universe based on results of non-specific filtering
affyUniverse <- featureNames(nsFiltered)
entrezUniverse <- unlist(mget(affyUniverse, hgu95av2ENTREZID))
if (any(duplicated(entrezUniverse)))
  stop("error in gene universe: can't have duplicate Entrez Gene Ids")

## Also define an alternate universe based on the entire chip
chipAffyUniverse <- featureNames(bcrAblOrNeg)
chipEntrezUniverse <- mget(chipAffyUniverse, hgu95av2ENTREZID)
chipEntrezUniverse <- unique(unlist(chipEntrezUniverse)) 
```

**Summary of non-specific filtering:** Our non-specific filtering
procedure removed probes missing either Entrez Gene identifies or
mappings to GO terms.  Because of an imbalance of men and women by
group, probes measuring genes on the Y chromosome were dropped.  The
inter-quartile range was used with a cutoff of `r iqrCutoff` to
select probes with sufficient variability across samples to be
informative; probes with little variability across all samples are
inherently uninteresting.  Finally, the set of remaining probes was
refined by ensuring that each probe maps to exactly one Entrez Gene
identifier.  For those probes mapping to the same Entrez Gene ID, the
probe with largest IQR was selected.

Producing a set of Entrez Gene identifiers that map to a unique set of
probes at the non-specific filtering stage is important because genes
are mapped to GO categories using Entrez Gene IDs and we want to avoid
double counting any GO categories.  In all, the filtering left
`r length(featureNames(nsFiltered))` genes.

## Gene selection via t-test

We apply a standard $t$-test to identify a set of genes with
differential expression between the `r subsetType` and NEG groups.

```{r parametric1, echo=TRUE, results='hide', cache=TRUE}
ttestCutoff <- 0.05
ttests = rowttests(nsFiltered, "mol.biol")

smPV = ttests$p.value < ttestCutoff

pvalFiltered <- nsFiltered[smPV, ] 
selectedEntrezIds <- unlist(mget(featureNames(pvalFiltered), 
                                 hgu95av2ENTREZID))
```

There are `r sum(smPV)` genes with $p$-values less than `r ttestCutoff`.
We do not make use of any $p$-value correction methods since we are 
interested in a relatively long gene list.

A detail often omitted from GO association analyses is the fact that the
$t$-test, and most similar statistics, are directional. For a given
gene, average expression might be higher in the `r subsetType` group than 
in the NEG group, whereas for a different gene it might be the NEG group that 
shows the increased expression. By only looking at the $p$-values for the test
statistics, the directionality is lost. The danger is that an
association with a GO category may be found where the genes are not
differentially expressed in the same direction. One way to tackle this
problem is by separating the selected gene list into two lists according
to direction and running two analyses. A more elegant approach is the
subject of further research.

## Inputs

Often one wishes to perform many similar analyses using slightly
different sets of parameters and to facilitate this pattern of usage the
main interface to the Hypergeometric tests, `hyperGTest`, takes a single
parameter object as its argument. This argument is an instance of class
*GOHyperGParams*. There are also parameter classes *KEGGHyperGParams*
and *PFAMHyperGParams* defined in the `r Biocpkg("Category")` package
that allow for testing for association with KEGG pathways and PFAM
protein domains, respectively.

Using a parameter class instead of individual arguments makes it easier
to organize and execute a series of related analyses. For example, one
can create a list of *GOHyperGParams* instances and perform the
Hypergeometric test on each using R's `lapply` function:

````{r eval=FALSE}
resultList <- lapply(lisOfParamObjs, hyperGTest)
````

In the absence of a parameter class, this could be achieved using
`mapply`, but the result would be less readable. Because parameter
objects can be copied and modified, they tend to reduce duplication of
code. We'll demonstrate this in the following example.

Below, we create a parameter instance by specifying the gene list, the
universe, the name of the annotation data package, and the GO ontology
we wish to interrogate. For the example analysis, we have stored the
vector of Entrez Gene identifiers making up the gene universe in
`entrezUniverse`. The selected genes are stored in `selectedEntrezIds`.
If you are following along with your own data and have an
*ExpressionSet* instance resulting from a non-specific filtering
procedure, you can create the `entrezUniverse` and `selectedEntrezIds`
vectors using code similar to that shown here:

```{r withYourData1, eval=FALSE}
entrezUniverse <- unlist(mget(featureNames(yourData), 
                              hgu95av2ENTREZID)) 
if (any(duplicated(entrezUniverse))) 
 stop(\"error in gene universe: can't have duplicate Entrez Gene Ids")

pvalFiltered <- yourData[hasSmallPvalue, ]
selectedEntrezIds <-unlist(mget(featureNames(pvalFiltered),
                                hgu95av2ENTREZID))
```

Here is a description of all the arguments needed to construct a
*GOHyperGParams* instance.

**geneIds** A vector of gene identifiers that defines the selected list of genes. 
This is often the output of a test for differential expression  among two sample
groups. For experiments using Affymetrix expression arrays, this should be a 
vector of Entrez Gene IDs. If you are using the **YEAST** annotation package, 
the vector will consist of yeast systematic names.

**universeGeneIds** A vector of gene identifiers that defines the universe of 
possible genes. We recommend using the set of gene IDs that result from
non-specific filtering.The identifiers should be of the same type as the
`geneIds`; for Affymetrix arrays, these will be Entrez Gene IDs.

**annotation** A string giving the name of the annotation data package that 
corresponds to the chip used in the experiment.

**ontology** A two-letter string specifying one of the three GO ontologies: BP,
CC, or MF. The `hyperGTest` function only tests a single GO ontology at one time.

**pvalueCutoff** A numeric values between zero and one used as a $p$-value cutoff 
for $p$-values generated by the Hypergeometric test. When the test being 
performed is non-conditional, this is only used as a default value for printing
and summarizing the results. For a conditional analysis, the cutoff is used 
during the computation to determine perform the conditioning: child terms with 
a $p$-value less than `pvalueCutoff` are conditioned out of the test for their 
parent term.

**conditional** A logical value. If `TRUE`, the test performed uses the 
conditional algorithm. Otherwise, a standard Hypergeometric test is performed.
When `conditional(p) == TRUE`, the `hyperGTest` function uses the structure of 
the GO graph to estimate for each term whether or not there is evidence beyond 
that which is provided by the term's children to call the term in question 
statistically overrepresented.The algorithm conditions on all child terms that 
are themselves significant at the specified p-value cutoff. Given a subgraph of 
one of the three GO ontologies, the terms with no child categories are tested 
first. Next the nodes whose children have already been tested are tested. If any 
of a given node's children tested significant, the appropriate conditioning is 
performed.

**testDirection** A string which can be either "over" or "under". This determines 
whether the test performed detects over or under represented GO terms.

```{r standardHyperGeo, echo=TRUE, results='hide', cache=TRUE}
hgCutoff<- 0.001 
params <- new("GOHyperGParams", 
              geneIds=selectedEntrezIds,
              universeGeneIds=entrezUniverse, 
              annotation="hgu95av2.db",
              ontology="BP", 
              pvalueCutoff=hgCutoff, 
              conditional=FALSE,
              testDirection="over")
```

We would also like to perform a conditional test. Instead of having to
define a new *GOHyperGParams* instance by hand, we can create a copy and
update just the parameter of interest.

```{r condHyperGeo, echo=TRUE, results='hide', cache=TRUE}
paramsCond <- params 
conditional(paramsCond) <- TRUE 
```

A similar approach would work to create a parameter object for testing a
different GO ontology or to create an object for testing under
representation.

# Outputs and Result Summarization

The `hyperGTest` function returns an instance of class *GOHyperGResult*.
When the input parameter object is a *KEGGHyperGParams* or
*PFAMHyperGParams* instance, the result will instead be a *HyperGResult*
object. Most of the reporting and summarization methods demonstrated
here will work the same, except for those that deal specifically with GO
or the GO graph.

As shown below, printing the result at the R prompt provides a brief
summary of the test performed and the number of significant terms found.
Depending on how you pre-processed your gene list and gene universe, The
hyperGTest function may have to do even more filtering on both of these
for you. Genes that are not marked with a GO term from the ontology that
you specified will have to be discarded, and so you might notice that
your gene list and gene universe had shrank somewhat when you print the
results.

```{r standardHGTEST, cache=TRUE, echo=TRUE, results='hide'}
hgOver <- hyperGTest(params)
hgCondOver <- hyperGTest(paramsCond)
```

```{r HGTestAns}
hgOver 
hgCondOver
```

The `summary` function returns a *data.frame* summarizing the result. By
default, only the results for terms with a $p$-value less than the
cutoff specified in the parameter instance will be shown. However, you
can set a new cutoff using the `pvalue` argument. You can also set a
minimum number of genes for each term using the `categorySize` argument.
For *GOHyperGResult* objects, the `summary` method also has a
`htmlLinks` argument. When `TRUE`, the GO term names are printed as HTML
links to the GO website. 

```{r summaryEx}
df <- summary(hgOver)
names(df) # the columns 
dim(summary(hgOver, pvalue=0.1))
dim(summary(hgOver, categorySize=10)) 
```

Now we demonstrate some of the accessor functions that can be used to
extract detail from a result object. These functions are all detailed in
their respective manual pages.

```{r resultAccessors}
pvalues(hgOver)[1:3]

oddsRatios(hgOver)[1:3]

expectedCounts(hgOver)[1:3]

geneCounts(hgOver)[1:3]
universeCounts(hgOver)[1:3]

length(geneIds(hgOver)) 
length(geneIdUniverse(hgOver)[[3]])

## GOHyperGResult _only_ 
## (NB: edges go from parent to child)
goDag(hgOver)

geneMappedCount(hgOver) 
universeMappedCount(hgOver)

conditional(hgOver) 
testDirection(hgOver)
testName(hgOver)
```

To make it easy for non-technical users to review the results, the
`htmlReport` function generates an HTML file that can be viewed in any
web browser. The output generated by `htmlReport` as called below is
output to your current working directory.

```{r htmlReportExample, results='hide', echo=TRUE}
htmlReport(hgCondOver, file="ALL_hgco.html")
```

```{r helperFunc, echo=FALSE, results='hide', cache=TRUE}
sigCategories <- function(res, p){
  if (missing(p)) 
    p <- pvalueCutoff(res) 
  pv <- pvalues(res) 
  goIds <- names(pv[pv < p])
  goIds 
}
```

```{r plotFuns, echo=FALSE, results='hide', cache=TRUE}
coloredGoPlot <- function(ccMaxGraph, hgOver, hgCondOver) {
  nodeColors <- sapply(nodes(ccMaxGraph),
                       function(n) {
                         if (n %in% sigCategories(hgCondOver))
                             "dark red"
                          else if (n %in% sigCategories(hgOver))
                            "pink" 
                          else 
                            "gray"
                        }) 
 nattr <- makeNodeAttrs(ccMaxGraph,
                        label=nodes(ccMaxGraph), 
                        shape="ellipse", 
                        fillcolor=nodeColors,
                        fixedsize=FALSE)
 plot(ccMaxGraph, nodeAttrs=nattr)
}

getMaxConnCompGraph <- function(hgOver, hgCondOver){
  ##uGoDagRev \<-ugraph(goDag(hgOver)) 
  sigNodes <- sigCategories(hgOver) 
  ##adjNodes \<- unlist(adj(uGoDagRev, sigNodes))
  adjNodes <- unlist(adj(goDag(hgOver),sigNodes))
  displayNodes <- unique(c(sigNodes, adjNodes)) 
  displayGraph <- subGraph(displayNodes, goDag(hgOver))
  cc <- connComp(displayGraph)
  ccSizes <- listLen(cc) 
  ccMaxIdx <- which(ccSizes == max(ccSizes))
  ccMaxGraph <- subGraph(cc[[ccMaxIdx]], displayGraph)
  ccMaxGraph
}

```

# GOstats Capabilities

In the Hypergeometric model, each term is treated as an independent
classification. Each gene is cross-classified according to whether or
not it has been selected and whether or not it is annotated, not
necessarily specifically annotated, at a particular term. A
Hypergeometric probability is computed to assess whether the number of
selected genes associated with the term is larger than expected.

The `hyperGTest` function provides an implementation of the commonly
applied Hypergeometric calculation for over or under-representation of
GO terms in a specified gene list. This computation ignores the
structure of the GO terms, treating each term as independent from all
other terms.

Often an analysis for GO term associations results in the identification
of directly related GO terms with considerable overlap of genes. This is
because each GO term inherits all annotations from its more specific
descendants. To alleviate this problem, we have implemented a method
which conditions on all child terms that are themselves significant at a
specified $p$-value cutoff. Given a subgraph of one of the three GO
ontologies, we test the leaves of the graph, that is, those terms with
no child terms. Before testing the terms whose children have already
been tested, we remove all genes annotated at significant children from
the parent's gene list. This continues until all terms have been tested.

```{r info}
sessionInfo()
```

# References

\name{shortestPath}
\alias{shortestPath}
\title{ Shortest Path Analysis }
\description{
 The shortest path analysis was proposed by Zhou et. al. The basic
 computation is to find the shortest path in a supplied graph between
 two Entrez Gene IDs. Zhou et al claim that other genes annotated along
 that path are likely to have the same GO annotation as the two end
 points.
}
\usage{
shortestPath(g, GOnode, mapfun=NULL, chip=NULL)
}
\arguments{
  \item{g}{An instance of the \code{graph} class. }
  \item{GOnode}{A length one character vector specifying the GO node of
    interest. }

    \item{mapfun}{A function taking a character vector of GO IDs as its
    only argument and returning a list of character vectors of Enterz
    Gene IDs annotated at each corresponding GO ID.  The function should
    behave similarly to \code{mget(x, go2egmap, ifnotfound=NA)}, that
    is, \code{NA} should be returned if a specified GO ID has no Entrez
    ID mappings.  See details for the interaction of \code{mapfun} and
    \code{chip}.}

  \item{chip}{The name of a DB-based annotation data package (the name
    will end in ".db").  This package will be used to generate an Entrez
    ID to GO ID mapping instead of \code{mapfun}.}
}
\details{
  The algorithm implemented here is quite simple. All Entrez Gene 
  identifiers that are annotated at the GO node of interest are
  obtained. Those that are found as nodes in the graph are retained and
  used for the computation. For every pair of nodes at the GO term the
  shortest path between them is computed using \code{sp.between} from
  the RBGL package.

  There is a presumption that the graph is \code{undirected}. This
  restriction could probably be lifted if there was some reason for it -
  a patch would be gratefully accepted.

  The mapping of GO node to Entrez ID is achieved in one of three ways:

  \enumerate{
    \item If \code{mapfun} is provided, it will be used to perform the
      needed lookups.  In this case, \code{chip} will be ignored.

    \item If \code{chip} is provided and \code{mapfun=NULL}, then the
      needed lookups will be done based on the GO to Entrez mappings
      encapsulated in the specified annotation data package.  This is
      the recommended usage.

    \item If \code{mapfun} and \code{chip} are \code{NULL} or missing,
      then the function will attempt to load the GO package (the
      environment-based package, distinct from GO.db).  This package
      contains a legacy environment mapping GO IDs to Entrez IDs.  If
      the GO package is not available, an error will be raised.
      Omitting both \code{mapfun} and \code{chip} is not recommended as
      it is not compatible with the DB-based annotation data packages.
  }
  
}
\value{
  The return values is a list with the following components:
  \item{shortestpaths }{A list of the ouput from \code{sp.between}. The
    names are the names of the nodes used as the two endpoints}
  \item{nodesUsed }{A vector of the Entrez Gene IDs that were both found
    at the GO term of interest and were nodes in the supplied graph,
    \code{g}. These were used to compute the shortest paths.}
  \item{nodesNotUsed}{A vector of Entrez Gene IDs that were annotated at
    the GO term, but were not found in the graph \code{g}.}
}
\references{Transitive functional annotation by shortest-path analysis
  of gene expression data, by X. Zhou and M-C J. Kao and W. H. Wong,
  PNAS, 2002}

\author{R. Gentleman }

\seealso{\code{\link[RBGL]{sp.between}}}

\examples{
library("hgu95av2.db")
library("RBGL")

set.seed(321)
uniqun <- function(x) unique(unlist(x))

goid <- "GO:0005778"
egIds <- uniqun(mget(uniqun(hgu95av2GO2PROBE[[goid]]),
                            hgu95av2ENTREZID))

v1 <- randomGraph(egIds, 1:10, .3, weights=FALSE)
## Since v1 is random, it might be disconnected and we need a
## connected graph to guarantee the existence of a path.
c1 <- connComp(v1)
largestComp <- c1[[which.max(sapply(c1, length))]]
v2 <- subGraph(largestComp, v1)

a1 <- shortestPath(v2, goid, chip="hgu95av2.db")

}
\keyword{ manip }

\name{oneGOGraph}
\alias{oneGOGraph}
\alias{GOGraph}

\title{Construct the GO graph given a set of leaves.  }

\description{
  Given one or more GO identifiers (which indicate the leaves in the
  graph) and a set of mappings to the less specific sets of nodes this
  function will construct the graph that includes that node and all
  children down to the root node for the ontology.
}
\usage{
oneGOGraph(x, dataenv)
GOGraph(x, dataenv)

}
\arguments{
  \item{x}{A character vector of GO identifiers. }
  \item{dataenv}{ An environment for finding the parents of that
      term. }


}
\details{
  For any set of GO identifiers (from a common ontology) we define the
  induced GO graph to be  that graph, based on 
  the DAG structure (child - parent) of the GO ontology of terms, which
  takes the most specific set of GO terms that apply (for that
  ontology) and then joins these to all less specific terms. These
  functions help construct such graphs.
}
\value{
  The induced GO graph (or NULL) for the given GO identifier.
}
\author{R. Gentleman}

\seealso{\code{\link{makeGOGraph}}}

\examples{

 library("GO.db")
 g1 <- oneGOGraph("GO:0003680", GOMFPARENTS)
 g2 <- oneGOGraph("GO:0003701", GOMFPARENTS)
 g3 <- join(g1, g2)

 g4 <- GOGraph(c("GO:0003680", "GO:0003701"), GOMFPARENTS)
if( require("Rgraphviz") && interactive() )
  plot(g3)
}
\keyword{manip}

\name{idx2dimnames}
\alias{idx2dimnames}
\title{Index to Dimnames}
\description{
  A function to map from integer offsets in an array to the
corresponding values of the row and column names. There is probably a
better way but I didn't find it.
}
\usage{
idx2dimnames(x, idx)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{a \code{matrix} or \code{data.frame}. }
  \item{idx}{An integer vector of offsets into the matrix (values
  between 1 and the \code{length} of the matrix.}
}

\value{
 A list with two components.
  If it is a LIST, use
  \item{rowNames }{The row names corresponding to the integer index.}
  \item{colNames }{The column names corresponding to the integer index.}
}

\author{R. Gentleman}

\seealso{\code{\link[base:dimnames]{dimnames}} }
\examples{
 data(Ndists)
 ltInf = is.finite(Ndists)
 xx = idx2dimnames(Ndists, ltInf)
}
\keyword{manip}

\name{probeSetSummary}
\alias{probeSetSummary}

\title{Summarize Probe Sets Associated with a hyperGTest Result}

\description{
  Given the result of a \code{hyperGTest} run (an instance of
  \code{GOHyperGResult}), this function lists all Probe Set IDs
  associated with the selected Entrez IDs annotated at each
  significant GO term in the test result.
}

\usage{
probeSetSummary(result, pvalue, categorySize, sigProbesets, ids = "ENTREZID")
}

\arguments{
  \item{result}{A \code{GOHyperGResult} instance.  This is the output   
      of the \code{hyperGTest} function when testing the GO category.}
  \item{pvalue}{Optional p-value cutoff.  Only results for GO terms
      with a p-value less than the specified value will be returned.
      If omitted, \code{pvalueCutoff(result)} is used.}
  \item{categorySize}{Optional minimum size (number of annotations)
      for the GO terms.  Only results for GO terms with
      \code{categorySize} or more annotations will be returned.  If
      omitted, no category size criteria will be used.}
  \item{sigProbesets}{Optional vector of probeset IDs. See details for
    more information.}
  \item{ids}{Character. The type of IDs used in creating the
  \code{GOHyperGResult} object. Usually 'ENTREZID', but may be e.g.,
  'ACCNUM' if using A. thaliana chip.}
}

\details{
  Usually the goal of doing a Fisher's exact test on a set of
  significant probesets is to find pathways or cellular activities that
  are being perturbed in an experiment. After doing the test, one
  usually gets a list of significant GO terms, and the next logical step
  might be to determine which probesets contributed to the significance
  of a certain term.

  Because the input for the Fisher's exact test consists of a vector of
  unique Entrez Gene IDs, and there may be multiple probesets that
  interrogate a particular transcript, the ouput for this function lists
  all of the probesets that map to each Entrez Gene ID, along with an
  indicator that shows which of the probesets were used as input.

  The rationale for this is that one might not be able to assume a given
  probeset actually interrogates the intended transcript, so it might be
  useful to be able to check to see what other similar probesets are
  doing.

  Because one of the first steps before running \code{hyperGTest} is to
  subset the input vectors of geneIds and universeGeneIds, any
  information about probeset IDs that interrogate the same gene
  transcript is lost. In order to recover this information, one can pass
  a vector of probeset IDs that were considered significant. This vector
  will then be used to indicate which of the probesets that map to a
  given GO term were significant in the original analysis.
}

\value{
  A \code{list} of \code{data.frame}.  Each element of the list
  corresponds to one of the GO terms (the term is provides as the name
  of the element).  Each \code{data.frame} has three columns:
  the Entrez Gene ID (\code{EntrezID}), the probe set ID
  (\code{ProbeSetID}), and a 0/1 indicator of whether the probe set ID
  was provided as part of the initial input (\code{selected})

  Note that this 0/1 indicator will only be correct if the 'geneId'
  vector used to construct the \code{GOHyperGParams} object was a named
  vector (where the names are probeset IDs), or if a vector of
  'sigProbesets' was passed to this function.
}

\author{S. Falcon and J. MacDonald}

\examples{
  ## Fake up some data
  library("hgu95av2.db")
  library("annotate")
  prbs <- ls(hgu95av2GO)[1:300]
  ## Only those with GO ids
  hasGO <- lengths(lapply(mget(prbs, hgu95av2GO), names)) != 0
  prbs <- prbs[hasGO]
  prbs <- getEG(prbs, "hgu95av2")
  ## remove duplicates, but keep named vector
  prbs <- prbs[!duplicated(prbs)]
  ## do the same for universe
  univ <- ls(hgu95av2GO)[1:5000]
  hasUnivGO <- lengths(lapply(mget(univ, hgu95av2GO), names)) != 0
  univ <- univ[hasUnivGO]
  univ <- unique(getEG(univ, "hgu95av2"))

  p <- new("GOHyperGParams", geneIds=prbs, universeGeneIds=univ,
  ontology="BP", annotation="hgu95av2", conditional=TRUE)
  ## this part takes time...
  if(interactive()){
    hyp <- hyperGTest(p)
    ps <- probeSetSummary(hyp, 0.05, 10)
  }
}

\keyword{manip}
\keyword{htest}

\name{Ndists}
\alias{Ndists}
\alias{Bdists}
\docType{data}
\title{Distance matrices for the BCR/ABL and NEG subgroups.}
\description{
  These are precomputed distance matrices between all transcription
  factors selected. In the future they will be computed on the fly but
  currently that takes about 3 hours and so precomputed versions are
  supplied. 
}
\usage{data(Ndists)
  data(Bdists)}
\format{
  These are both distance matrices.
}
\source{
 They are based on the ALL data, \code{\link[ALL]{ALL}}.
}
\examples{
data(Ndists)
data(Bdists)
}
\keyword{datasets}

\name{makeGOGraph}
\alias{makeGOGraph}
\title{ Construct a GO Graph }
\description{
  The directed acyclic graph (DAG) based on finding the most specific
  terms for the supplied Entrez Gene IDs is constructed and returned. The
  constructuion is per GO ontology (there are three, MF, BP and CC) and
  once the most specific terms have been identified then all less
  specific terms are found (these are the parents of the terms) and then
  their parents and so on, until the root is encountered.
}
\usage{
makeGOGraph(x, Ontology = "MF", removeRoot = TRUE, mapfun = NULL,
            chip = NULL)
}
\arguments{
  \item{x}{A vector of Entrez Gene IDs. }
  \item{Ontology}{Which GO ontology to use (CC, BP, or MF). }
  \item{removeRoot}{A logical value indicating whether the GO root node
    should be removed or not.}

  \item{mapfun}{A function taking a character vector of Entrez Gene IDs
    as its only argument and returning a list of "GO lists" matching the
    structure of the lists in the GO maps of annotation data packages.
    The function should behave similarly to
    \code{mget(x, eg2gomap, ifnotfound=NA)},
    that is, \code{NA} should be returned if a specified Entrez ID has
    no GO mapping.  See details for the interaction of \code{mapfun} and
    \code{chip}.}

  \item{chip}{The name of a DB-based annotation data package (the name
    will end in ".db").  This package will be used to generate an Entrez
    ID to GO ID mapping instead of \code{mapfun}.}

}

\details{
  For each supplied Entrez Gene identifier all the GO annotations (in the
  specified ontology) are found.  The mapping is achieved in one of
  three ways:

  \enumerate{
    \item If \code{mapfun} is provided, it will be used to perform the
      needed lookups.  In this case, \code{chip} will be ignored.

    \item If \code{chip} is provided and \code{mapfun=NULL}, then the
      needed lookups will be done based on the Entrez to GO mappings
      encapsulated in the specified annotation data package.  This is
      the recommended usage.

    \item If \code{mapfun} and \code{chip} are \code{NULL} or missing,
      then the function will attempt to load the GO package (the
      environment-based package, distinct from GO.db).  This package
      contains a legacy environment mapping Entrez IDs to GO IDs.  If
      the GO package is not available, an error will be raised.
      Omitting both \code{mapfun} and \code{chip} is not recommended as
      it is not compatible with the DB-based annotation data packages.
  }

  The mappings are different for the different ontologies. Typically a
  GO indentifier is used only in one specific ontology.

  The resulting structure is stored in a graph using the \code{graph}
  package, again from Bioconductor.

}
\value{
  An object that inherits from the \code{graph} class. The particular
  implementation is not specified.
}
\references{The Gene Ontology Consortium }
\author{R. Gentleman}

\seealso{\code{\link{oneGOGraph}}}

\examples{
 library("hgu95av2.db")
 set.seed(321)
 gN <- unique(sample(keys(hgu95av2.db, 'ENTREZID'), 4))
 gg1 <- makeGOGraph(gN, "BP", chip="hgu95av2.db")

}
\keyword{manip}

\name{hyperGTest}
\alias{hyperGTest}
\alias{hyperGTest,GOHyperGParams-method}
\alias{hyperGTest,OBOHyperGParams-method}

\title{Hypergeometric Tests for GO term association}

\description{ Given a \code{GOHyperGParams} instance
  containing a set of unique Entrez Gene Identifiers, a microarray
  annotation data package name, and the GO ontology of interest, this
  function will compute Hypergeomtric p-values for over or
  under-representation of each GO term in the specified ontology among
  the GO annotations for the interesting genes.  The computations can
  be done conditionally based on the structure of the GO graph.}


\arguments{
  \item{p}{A \code{GOHyperGParams} or \code{OBOHyperGParams} instance }
}

\details{

  When \code{conditional(p) == TRUE}, the \code{hyperGTest} function
  uses the structure of the GO graph to estimate for each term whether
  or not there is evidence beyond that which is provided by the term's
  children to call the term in question statistically overrepresented.

  The algorithm conditions on all child terms that are themselves
  significant at the specified p-value, odds ratio, minimum or
  maximum gene set size cutoff.  Given a subgraph of
  one of the three GO ontologies, or the ontology given in the
  \code{OBOHyperGParams} instance, the terms with no child categories
  are tested first.  Next the nodes whose children have already been
  tested are tested.  If any of a given node's children tested
  significant, the appropriate conditioning is performed.
}
\value{
  A \code{GOHyperGResult} or \code{OBOHyperGResult} instance.
}

\references{FIXME}

\author{Seth Falcon}
\seealso{
  \code{\link{GOHyperGResult-class}},
  %\code{\link[Category]{geneCategoryHyperGeoTest}},
  \code{\link[Category]{geneGoHyperGeoTest}},
  \code{\link[Category]{geneKeggHyperGeoTest}}
}
\keyword{models}

\name{GOstats-defunct}
\alias{GOstats-defunct}
\title{Defunct Functions in GOstats Package}
%------ PLEASE: put \alias{.} here for  EACH !
\alias{combGOGraph}
\alias{hyperGtable}
\alias{hyperG2Affy}
\alias{GOLeaves}
\alias{selectedGenes}
\alias{GOHyperG}
\alias{GOKEGGHyperG}
\alias{getGoGraph}

\description{
  The functions or variables listed here are no longer part of GOstats as
  they are not needed (any more).
}
\usage{
combGOGraph()
hyperGtable()
hyperG2Affy()
selectedGenes()
GOHyperG()
GOKEGGHyperG()
getGoGraph()
}
\details{
  \code{combGOGraph} was replaced by \code{join}.
  \code{hyperGtable} was replaced by \code{summary}.
  \code{hyperG2Affy} was replaced by \code{probeSetSummary}.
  \code{GOLeaves} was replaced by \code{graph::leaves}.
  \code{selectedGenes} was replaced by \code{geneIdsByCategory}.
  \code{GOHyperG} was replaced by \code{hyperGTest}.
  \code{GOKEGGHyperG} was replaced by \code{hyperGTest}.
  \code{getGoGraph} was replaced by \code{GOGraph}.
}

\keyword{manip}

\name{GOHyperGResult-class}
\docType{class}
\alias{GOHyperGResult-class}
\alias{geneCounts,GOHyperGResult-method}

\alias{goDag}
\alias{goDag,GOHyperGResult-method}

\alias{oddsRatios,GOHyperGResult-method}

\alias{expectedCounts,GOHyperGResult-method}

\alias{pvalues,GOHyperGResult-method}

\alias{geneIdUniverse,GOHyperGResult-method}

\alias{htmlReport,GOHyperGResult-method}

\alias{summary,GOHyperGResult-method}

\alias{conditional,GOHyperGResult-method}

\title{Class "GOHyperGResult"}

\description{	

  This class represents the results of a test for overrepresentation
  of GO categories among genes in a selected gene set based upon the
  Hypergeometric distribution.

  For details on extracting information from this object, be sure to
  read the accessor documentation in the Category package:
  \link[Category]{HyperGResult-accessors}.

}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("GOHyperGResult", ...)}.
}

\section{Slots}{
	 \describe{
    \item{\code{goDag}:}{Object of class \code{"graph"} representing
        the DAG of GO terms tested.}
    \item{\code{pvalue.order}:}{Object of class \code{"integer"}.
        The sort order of the computed p-values.}
    \item{\code{annotation}:}{Object of class \code{"character"}.  The
      name of the annotation data package used in the analysis.}
    \item{\code{geneIds}:}{Object of class \code{"ANY"}.  The
        intersection of the gene identifiers given as input and the
        computed gene universe.}
    \item{\code{testName}:}{Object of class \code{"character"}.
        Identifies the testing method used to produce this result instance.}
    \item{\code{pvalueCutoff}:}{Object of class \code{"numeric"}.
        The cutoff for significance used for some testing methods.
        Also used for pretty display in the \code{show} method.}
    \item{\code{conditional}:}{A logical indicating whether the
      calculation should condition on the GO structure.}
    \item{\code{testDirection}:}{A string which can be either "over"
      or "under". This determines whether the test performed detects
      over or under represented GO terms.}
  }
}

\section{Extends}{
Class \code{"HyperGResultBase"}, directly.
}
\section{Methods}{
  \describe{
    \item{goDag}{\code{signature(r = "GOHyperGResult")}:
        return the \code{graph} instance representing the DAG of the
        GO terms that were tested.}


  \item{summary}{\code{signature(r = "GOHyperGResult")}: Returns a
      \code{data.frame} summarizing the test result.  Optional
      arguments \code{pvalue} and \code{categorySize} allow
      specification of maximum p-value and minimum categorySize,
      respectively.  Optional argument \code{htmlLinks} is a logical
      value indicating whether to add HTML links (useful in
      conjunction with xtables print method with \code{type} set to
      \code{"html"}).}

  \item{htmlReport}{\code{signature(r = "GOHyperGResult")}: Write an
      HTML version of the table produced by the \code{summary} method.
      The path of a file to write the report to can be specified using
      the \code{file} argument.  The default is \code{file=""} which
      will cause the report to be printed to the screen.  If you wish
      to create a single report comprising multiple results you can
      set \code{append=TRUE}.  The default is \code{FALSE} (overwrite
      preexisting report file).  You can specify a string to use as an
      identifier for each table by providing a value for the
      \code{label} argument.  Additional named arguments will be
      passed to the \code{summary} method.}

  \item{description}{\code{signature(object = "GOHyperGResult")}:
      Return a string giving a one-line description of the result.}

  }


}

\author{Seth Falcon}

\seealso{
  \link[Category]{HyperGResult-accessors}
}

\keyword{classes}

\name{compCorrGraph}
\alias{compCorrGraph}
\title{A function to compute a correlation based graph from Gene
Expression Data }
\description{
  Given a set of gene expression data (an instance of the
  \code{ExpressionSet} class) this function computes a graph based on
  correlations between the probes.
}
\usage{
compCorrGraph(eSet, k = 1, tau = 0.6)
}
\arguments{
  \item{eSet}{An instance of the \code{ExpressionSet} class. }
  \item{k}{The power to raise the correlations to. }
  \item{tau}{The lower cutoff for absolute correlations. }
}
\details{
  Zhou et al. describe a method of computing a graph between probes
  (genes) based on estimated correlations between probes. This function
  implements some of their methods.

  Pearson correlations between probes are computed and then these are
  raised to the power \code{k}. Any of the resulting estimates that are
  less than \code{tau} in absolute value are set to zero. 
}
\value{
  An instance of the \code{graph} class. With edges and edge weights
  determined by applying the algorithm described previously.
}
\references{Zhou et al., Transitive functional annotation by
shortest-path analysis of gene expression data.}
\author{R. Gentleman}

\seealso{\code{\link{compGdist}}}
\examples{

 ## Create an ExpressionSet to work with
 set.seed(123)
 exprMat <- matrix(runif(50 * 5), nrow=50)
 genData <- new("ExpressionSet", exprs=exprMat)

 corrG = compCorrGraph(genData)

}
\keyword{ manip }

\name{GOstats-package}
\alias{GOstats-package}
\alias{GOstats}
\docType{package}
\title{
Tools for manipulating GO and microarrays.
}
\description{
A set of tools for interacting with GO and microarray
data. A variety of basic manipulation tools for graphs,
hypothesis testing and other simple calculations.
}
\details{
\tabular{ll}{
Package: \tab GOstats\cr
Version: \tab 1.7.4\cr
Date: \tab 23-08-2006\cr
biocViews: \tab Statistics, Annotation, GO, MultipleComparisons\cr
Depends: \tab graph (>= 1.9.25), GO, annotate, RBGL, xtable, Biobase,
genefilter, multtest, Category (>= 1.3.7), methods\cr
Imports: \tab methods, Category\cr
Suggests: \tab hgu95av2.db (>= 1.6.0)\cr
License: \tab Artistic\cr
}

Index:
\preformatted{
ALL                     Acute Lymphoblastic Leukemia Data from the Ritz
                        Laboratory
GOstats-defunct         Defunct Functions in GOstats Package
Ndists                  Distance matrices for the BCR/ABL and NEG
                        subgroups.
compCorrGraph           A function to compute a correlation based graph
                        from Gene Expression Data
compGdist               A function to compute the distance between
                        pairs of nodes in a graph.
dropECode               Drop GO labels for specified Evidence Codes
getEvidence             Get the Evidence codes for a set of GO terms.
getGOTerm               Functions to Access GO data.
getOntology             Get GO terms for a specified ontology
hasGOannote             Check for GO annotation
idx2dimnames            Index to Dimnames
makeGOGraph             Construct a GO Graph
notConn                 Find genes that are not connected to the
                        others.
oneGOGraph              Construct the GO graph given a set of leaves.
shortestPath            Shortest Path Analysis
simLL                   Functions to compute similarities between GO
                        graphs and also between Entrez Gene IDs based on
                        their induced GO graphs.
triadCensus             Triad Functions
}

Further information is available in the following vignettes:
\tabular{ll}{
\code{GOstats} \tab Using GOstats (source, pdf)\cr
\code{GOusage} \tab Basic GO Usage (source, pdf)\cr
\code{GOvis} \tab Visualizing Data Using GOstats (source, pdf)\cr
}

}
\author{
R. Gentleman with contributions from S. Falcon

Maintainer: R. Gentleman <rgentlem@fhcrc.org>
}

\keyword{ package }

\name{termGraphs}
\alias{termGraphs}
\alias{inducedTermGraph}
\alias{plotGOTermGraph}

\title{Extraction and Plotting of GO Terms from a GOHyperGResult Object}
\description{
  These functions extract and plot \code{graph} instances representing the
  relationships among GO terms tested using \code{hyperGTest}.
}
\usage{
termGraphs(r, id = NULL, pvalue = NULL, use.terms = TRUE)
inducedTermGraph(r, id, children = TRUE, parents = TRUE)
plotGOTermGraph(g, r = NULL, add.counts = TRUE, max.nchar = 20,
                node.colors=c(sig="lightgray", not="white"),
                node.shape="plaintext", ...)
}

\arguments{

  \item{r}{A \code{GOHyperGResult} object as returned by
    \code{hyperGTest} when given a \code{GOHyperGParams} object as
    input. }
  
  \item{id}{A character vector of category IDs that specifies which
    terms should be included in the graph.}

  \item{pvalue}{Numeric p-value cutoff to use for selecting category
    terms to include.  Will be ignored if \code{id} is present.}

  \item{use.terms}{Logical value indicating whether a \code{"term"} node
    attribute should be added to the returned graph providing the more
    descriptive, but possibly much longer, GO Terms.}

  \item{children}{A logical value indicating whether to include direct
    child terms of the terms specified by \code{id}.}

  \item{parents}{A logical value indicating whether to include direct
    parent terms of the terms specified by \code{id}.}

  \item{g}{A \code{graph} object as returned by \code{inducedTermGraph}
    or \code{termGraphs}.}

  \item{add.counts}{A logical value indicating whether category size
    counts should be added to the node labels when plotting.}

  \item{max.nchar}{The maximum character length for node labels in the plot.}

  \item{node.colors}{A named character vector of length two with
    compoents \code{sig} and \code{not}, giving color names for the
    significant and non-significant nodes, respectively.}

  \item{node.shape}{This argument controls the shape of the plotted
    nodes and must take on a value allowed by Rgraphviz.}

  \item{...}{For \code{plotGOTermGraph}, extra arguments are passed to
    the \code{plot} function.}

}

\details{
  \describe{
    
    \item{termGraphs}{returns a list of \code{graph} objects each
      representing one of the connected components of the subgraph of
      the GO ontology induced by selecting the specified GO IDs (if
      \code{id} is present) or by selecting the GO IDs that have a
      p-value less that \code{pvalue}.  If \code{use.terms} is
      \code{TRUE} the GO IDs will be translated into GO Term names and
      attached to the nodes as node attributes (see \code{nodeData}).
      Edges in the graphs go from child (more specific) to parent (less
      specific).}

    \item{inducedTermGraph}{returns a \code{graph} object representing
      the GO graph induced by the terms specified by \code{id}.  The
      \code{children} and \code{parent} arguments control whether direct
    children and/or direct parents of the terms specified by \code{id}
    are added to the graph (at least one of the two must be
    \code{TRUE}).}

    \item{plotGOTermGraph}{Create a plot using Rgraphviz of a
      \code{graph} object as returned by either \code{termGraphs} or
      \code{inducedTermGraph}.  If a \code{GOHyperGResult} object is
      provided, then the nodes will be colored according to significance
      (based on the result object's \code{pvalueCutoff}) and counts will
      be added to show the size of the categories.
    }

  }
}

\author{Seth Falcon}

\keyword{ manip }

\name{simLL}
\alias{simLL}
\alias{simUI}
\alias{simLP}
\title{Functions to compute similarities between GO graphs and also
between Entrez Gene IDs based on their induced GO graphs.}
\description{
  Both \code{simUI} and \code{simLP} compute a similarity measure
  between two GO graphs. For \code{simLL}, first the induced GO graph
  for each of its arguments is found and then these are passed to one
  of \code{simUI} or \code{simLP}.
}
\usage{
simLL(ll1, ll2, Ontology = "MF", measure = "LP", dropCodes = NULL,
      mapfun = NULL, chip = NULL)
simUI(g1, g2)
simLP(g1, g2)
}

\arguments{
  \item{ll1}{A Entrez Gene ID as a character vector. }
  \item{ll2}{A Entrez Gene ID as a character vector.}
  \item{Ontology}{Which ontology to use ("MF", "BP", "CC"). }
  \item{measure}{ Which measure to use ("LP", "UI"). }
  \item{dropCodes}{ A set of evidence codes to be ignored in
    constructing the induced GO graphs. }

  \item{mapfun}{A function taking a character vector of Entrez Gene IDs
    as its only argument and returning a list of "GO lists" matching the
    structure of the lists in the GO maps of annotation data packages.
    The function should behave similarly to \code{mget(x, eg2gomap,
      ifnotfound=NA)}, that is, \code{NA} should be returned if a
      specified
    Entrez ID has no GO mapping.  See details for the interaction of
    \code{mapfun} and \code{chip}.}

  \item{chip}{The name of a DB-based annotation data package (the name
    will end in ".db").  This package will be used to generate an Entrez
    ID to GO ID mapping instead of \code{mapfun}.}

  \item{g1}{An instance of the \code{graph} class.}
  \item{g2}{An instance of the \code{graph} class.}
}
\details{
  For each of \code{ll1} and \code{ll2} the set of most specific GO
  terms within the ontology specified (\code{Ontology}) that are not
  based on any excluded evidence code (\code{dropCodes}) are found.  The
  mapping is achieved in one of three ways:

  \enumerate{
    \item If \code{mapfun} is provided, it will be used to perform the
      needed lookups.  In this case, \code{chip} will be ignored.

    \item If \code{chip} is provided and \code{mapfun=NULL}, then the
      needed lookups will be done based on the Entrez to GO mappings
      encapsulated in the specified annotation data package.  This is
      the recommended usage.

    \item If \code{mapfun} and \code{chip} are \code{NULL} or missing,
      then the function will attempt to load the GO package (the
      environment-based package, distinct from GO.db).  This package
      contains a legacy environment mapping Entrez IDs to GO IDs.  If
      the GO package is not available, an error will be raised.
      Omitting both \code{mapfun} and \code{chip} is not recommended as
      it is not compatible with the DB-based annotation data packages.
  }

  Next, the induced GO graphs are computed.

  Finally these graphs are passed to one of \code{simUI}, (union
  intersection), or \code{simLP} (longest path). For \code{simUI} the
  distance is the size of the intersection of the node sets divided by
  the size of the union of the node sets. Large values indicate more
  similarity. These similarities are between 0 and 1.

  For \code{simLP} the length of the longest path in the intersection
  graph of the two supplied graph. Again, large values indicate more
  similarity. Similarities are between 0 and the maximum leaf depth of
  the graph for the specified ontology.
}
\value{
  A list with:
  \item{sim }{The numeric similarity measure.}
  \item{measure}{Which measure was used.}
  \item{g1}{The graph induced by \code{ll1}.}
  \item{g2}{The graph induced by \code{ll2}.}

  If one of the supplied Gene IDs does not have any GO terms associated
with it, in the selected ontology and with the selected evidence codes
then \code{NA} is returned.
}

\author{R. Gentleman }

\seealso{\code{\link{makeGOGraph}} }
\examples{
  library("hgu95av2.db")
  eg1 = c("9184", "3547")

  bb = simLL(eg1[1], eg1[2], "BP", chip="hgu95av2.db")
}
\keyword{manip}

\name{compGdist}
\alias{compGdist}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{A function to compute the distance between pairs of nodes in a graph. }
\description{
  Given a graph, \code{g}, and a set of nodes in the graph,
  \code{whNodes}, Dijkstra's shortest path algorithm is used to compute
  the distance between all pairs of nodes in \code{whNodes}.
}
\usage{
compGdist(g, whNodes, verbose = FALSE)
}
\arguments{
  \item{g}{ An instance of the \code{graph} class. }
  \item{whNodes}{A vector of lables of the nodes in \code{g} for which
  distances are to be computed. }
  \item{verbose}{If \code{TRUE} then output reporting the progress will
  be reported. }
}
\details{
  This function can be quite slow, computation of the pairwise
  distances is not especially fast and if \code{whNodes} is long then
  there are many of them to compute.
}
\value{
  A matrix containing the pairwise distances. It might be worth making
  this an instance of the \code{dist} class at some point.
}

\author{R. Gentleman }

\seealso{ \code{\link{compCorrGraph}} }
\examples{

 example(compCorrGraph)
 compGdist(corrG, nodes(corrG)[1:5])

}
\keyword{manip }

\name{triadCensus}
\alias{triadCensus}
\alias{triad}
\alias{enumPairs}
\alias{isTriad}
\alias{reduce2Degreek}
\title{ Triad Functions }
\description{
  These functions provide some tools for finding triads in an undirected
  graph. A triad is a clique of size 3. The function \code{triadCensus}
  returns a list of all triads.
}
\usage{
triadCensus(graph)
isTriad(x, y, z, elz, ely)
reduce2Degreek(graph, k)
enumPairs(iVec)

}
\arguments{
  \item{graph}{An instance of the \code{graph} class. }
  \item{k}{An integer indicating the minimum degree wanted.}
  \item{x}{A node}
  \item{y}{A node}
  \item{z}{A node}
  \item{elz}{The edgelist for \code{z}}
  \item{ely}{The edgelist for \code{y}}
  \item{iVec}{A vector of unique values}
}
\details{
  \code{enumPairs} takes a vector as input and returns a list of length
  \code{choose(length(iVec),2)/2} containing all unordered pairs of
  elements.

  \code{isTriad} takes three nodes as arguments. It is already known
  that \code{x} has edges to both \code{y} and \code{z} and we want to
  determine whether these are reciprocated. This is determined by
  examining \code{elz} for both \code{x} and \code{y} and then examining
  \code{ely} for both \code{x} and \code{z}. 

  \code{reduce2Degreek} is a function that takes an undirected graph as
  input and removes all nodes of degree less than \code{k}. This process
  is iterated until there are no nodes left (an error is thrown) or all
  nodes remaining have degree at least \code{k}. The resultant subgraph
  is returned. It is used here because to be in a triad all nodes must
  have degree 2 or more.

  \code{triadCensus} makes use of the helper functions described above
  and finds all triads in the graph.
  
}
\value{
  A list where each element is a triple indicating the members of the
  triad. Order is not important and all triads are reported in
  alphabetic order.
}

\author{R. Gentleman}
\note{See the graph package, RBGL and Rgraphviz for more details and
  alternatives. }


\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
}
\keyword{manip}

\name{notConn}
\alias{notConn}
\title{Find genes that are not connected to the others. }
\description{
  A function that takes as input a distance matrix and finds those
  entries that are not connected to any others (ie. those with distance
  \code{Inf}. 
}
\usage{
notConn(dists)
}
\arguments{
  \item{dists}{A distance matrix.}
}
\details{
  It is a very naive implementation. It presumes that not connected
  entries are not connected to any other entries, and this might not be
  true. Using the \code{connComp} function from the \code{graph}
  package or the \code{RBGL} package might be a better approach.
}
\value{
 A vector of the names of the items that are not connected.
}
\author{R. Gentleman }
\seealso{\code{\link[graph:graph-class]{connComp}}}
\examples{
 data(Ndists)
 notConn(Ndists)
}
\keyword{manip}
